@@ -55,14 +55,13 @@ import org.w3c.dom.Node;
  */
 public class CreateMerlinsExampleSixteen {
 
-    /** {@link org.apache.commons.logging} logging facility */
-    static org.apache.commons.logging.Log log = 
-        org.apache.commons.logging.LogFactory.getLog(CreateMerlinsExampleSixteen.class.getName());
+    static org.slf4j.Logger log =
+        org.slf4j.LoggerFactory.getLogger(CreateMerlinsExampleSixteen.class);
 
     static {
         org.apache.xml.security.Init.init();
     }
-    
+
     /**
      * Method main
      *
@@ -84,7 +83,7 @@ public class CreateMerlinsExampleSixteen {
 
         ks.load(fis, keystorePass.toCharArray());
 
-        PrivateKey privateKey = 
+        PrivateKey privateKey =
             (PrivateKey) ks.getKey(privateKeyAlias, privateKeyPass.toCharArray());
 
         if (privateKey == null) {
