@@ -20,30 +20,19 @@
  * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
  */
 /*
- * $Id: DOMStructure.java 1197150 2011-11-03 14:34:57Z coheigea $
+ * $Id: DOMStructure.java 1423915 2012-12-19 16:17:25Z coheigea $
  */
 package org.apache.jcp.xml.dsig.internal.dom;
 
 import javax.xml.crypto.MarshalException;
-import javax.xml.crypto.XMLStructure;
-import javax.xml.crypto.dom.DOMCryptoContext;
-import org.w3c.dom.Node;
+import javax.xml.crypto.XMLCryptoContext;
 
 /**
  * DOM-based abstract implementation of XMLStructure.
  *
  * @author Sean Mullan
  */
-public abstract class DOMStructure implements XMLStructure {
+public abstract class DOMStructure extends BaseStructure {
 
-    public final boolean isFeatureSupported(String feature) {
-        if (feature == null) {
-            throw new NullPointerException();
-        } else {
-            return false;
-        }
-    }
-
-    public abstract void marshal(Node parent, String dsPrefix, 
-        DOMCryptoContext context) throws MarshalException;
+    public abstract void marshal(XmlWriter xwriter, String dsPrefix, XMLCryptoContext context) throws MarshalException;
 }
