@@ -40,21 +40,20 @@ import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 
 /**
- * Allows selection of digital signature's algorithm, private keys, other 
+ * Allows selection of digital signature's algorithm, private keys, other
  * security parameters, and algorithm's ID.
  *
  * @author Christian Geuer-Pollmann
  */
 public class SignatureAlgorithm extends Algorithm {
 
-    /** {@link org.apache.commons.logging} logging facility */
-    private static org.apache.commons.logging.Log log = 
-        org.apache.commons.logging.LogFactory.getLog(SignatureAlgorithm.class);
+    private static org.slf4j.Logger log =
+        org.slf4j.LoggerFactory.getLogger(SignatureAlgorithm.class);
 
     /** All available algorithm classes are registered here */
-    private static Map<String, Class<? extends SignatureAlgorithmSpi>> algorithmHash = 
+    private static Map<String, Class<? extends SignatureAlgorithmSpi>> algorithmHash =
         new ConcurrentHashMap<String, Class<? extends SignatureAlgorithmSpi>>();
-   
+
     /** Field signatureAlgorithm */
     private final SignatureAlgorithmSpi signatureAlgorithm;
 
@@ -70,9 +69,9 @@ public class SignatureAlgorithm extends Algorithm {
     public SignatureAlgorithm(Document doc, String algorithmURI) throws XMLSecurityException {
         super(doc, algorithmURI);
         this.algorithmURI = algorithmURI;
-        
+
         signatureAlgorithm = getSignatureAlgorithmSpi(algorithmURI);
-        signatureAlgorithm.engineGetContextFromElement(this.constructionElement);
+        signatureAlgorithm.engineGetContextFromElement(getElement());
     }
 
     /**
@@ -88,12 +87,12 @@ public class SignatureAlgorithm extends Algorithm {
     ) throws XMLSecurityException {
         super(doc, algorithmURI);
         this.algorithmURI = algorithmURI;
-        
+
         signatureAlgorithm = getSignatureAlgorithmSpi(algorithmURI);
-        signatureAlgorithm.engineGetContextFromElement(this.constructionElement);
-        
+        signatureAlgorithm.engineGetContextFromElement(getElement());
+
         signatureAlgorithm.engineSetHMACOutputLength(hmacOutputLength);
-        ((IntegrityHmac)signatureAlgorithm).engineAddContextToElement(constructionElement);
+        ((IntegrityHmac)signatureAlgorithm).engineAddContextToElement(getElement());
     }
 
     /**
@@ -106,7 +105,7 @@ public class SignatureAlgorithm extends Algorithm {
     public SignatureAlgorithm(Element element, String baseURI) throws XMLSecurityException {
         this(element, baseURI, false);
     }
-    
+
     /**
      * Constructor SignatureAlgorithm
      *
@@ -118,47 +117,51 @@ public class SignatureAlgorithm extends Algorithm {
     public SignatureAlgorithm(
         Element element, String baseURI, boolean secureValidation
     ) throws XMLSecurityException {
-        super(element, baseURI);      
+        super(element, baseURI);
         algorithmURI = this.getURI();
-        
+
         Attr attr = element.getAttributeNodeNS(null, "Id");
         if (attr != null) {
             element.setIdAttributeNode(attr, true);
         }
-        
+
         if (secureValidation && (XMLSignature.ALGO_ID_MAC_HMAC_NOT_RECOMMENDED_MD5.equals(algorithmURI)
             || XMLSignature.ALGO_ID_SIGNATURE_NOT_RECOMMENDED_RSA_MD5.equals(algorithmURI))) {
             Object exArgs[] = { algorithmURI };
 
             throw new XMLSecurityException("signature.signatureAlgorithm", exArgs);
         }
-        
+
         signatureAlgorithm = getSignatureAlgorithmSpi(algorithmURI);
-        signatureAlgorithm.engineGetContextFromElement(this.constructionElement);
+        signatureAlgorithm.engineGetContextFromElement(getElement());
     }
 
     /**
      * Get a SignatureAlgorithmSpi object corresponding to the algorithmURI argument
      */
-    private static SignatureAlgorithmSpi getSignatureAlgorithmSpi(String algorithmURI) 
+    private static SignatureAlgorithmSpi getSignatureAlgorithmSpi(String algorithmURI)
         throws XMLSignatureException {
         try {
-            Class<? extends SignatureAlgorithmSpi> implementingClass = 
+            Class<? extends SignatureAlgorithmSpi> implementingClass =
                 algorithmHash.get(algorithmURI);
             if (log.isDebugEnabled()) {
                 log.debug("Create URI \"" + algorithmURI + "\" class \""
                    + implementingClass + "\"");
             }
-            return implementingClass.newInstance();   
+            if (implementingClass == null) {
+                Object exArgs[] = { algorithmURI };
+                throw new XMLSignatureException("algorithms.NoSuchAlgorithmNoEx", exArgs);
+            }
+            return implementingClass.newInstance();
         }  catch (IllegalAccessException ex) {
             Object exArgs[] = { algorithmURI, ex.getMessage() };
-            throw new XMLSignatureException("algorithms.NoSuchAlgorithm", exArgs, ex);
-        } catch (InstantiationException ex) {
+            throw new XMLSignatureException(ex, "algorithms.NoSuchAlgorithm", exArgs);
+        }  catch (InstantiationException ex) {
             Object exArgs[] = { algorithmURI, ex.getMessage() };
-            throw new XMLSignatureException("algorithms.NoSuchAlgorithm", exArgs, ex);
-        } catch (NullPointerException ex) {
+            throw new XMLSignatureException(ex, "algorithms.NoSuchAlgorithm", exArgs);
+        }  catch (NullPointerException ex) {
             Object exArgs[] = { algorithmURI, ex.getMessage() };
-            throw new XMLSignatureException("algorithms.NoSuchAlgorithm", exArgs, ex);
+            throw new XMLSignatureException(ex, "algorithms.NoSuchAlgorithm", exArgs);
         }
     }
 
@@ -235,12 +238,12 @@ public class SignatureAlgorithm extends Algorithm {
      * @param signingKey
      * @throws XMLSignatureException
      */
-    public void initSign(Key signingKey) throws XMLSignatureException {	   
+    public void initSign(Key signingKey) throws XMLSignatureException {	
         signatureAlgorithm.engineInitSign(signingKey);
     }
 
     /**
-     * Proxy method for {@link java.security.Signature#initSign(java.security.PrivateKey, 
+     * Proxy method for {@link java.security.Signature#initSign(java.security.PrivateKey,
      * java.security.SecureRandom)}
      * which is executed on the internal {@link java.security.Signature} object.
      *
@@ -288,14 +291,14 @@ public class SignatureAlgorithm extends Algorithm {
     public void initVerify(Key verificationKey) throws XMLSignatureException {
         signatureAlgorithm.engineInitVerify(verificationKey);
     }
-    
+
     /**
      * Proxy method for {@link java.security.Signature#verify(byte[])}
      * which is executed on the internal {@link java.security.Signature} object.
      *
      * @param signature
      * @return true if if the signature is valid.
-     * 
+     *
      * @throws XMLSignatureException
      */
     public boolean verify(byte[] signature) throws XMLSignatureException {
@@ -308,14 +311,14 @@ public class SignatureAlgorithm extends Algorithm {
      * @return the URI representation of Transformation algorithm
      */
     public final String getURI() {
-        return constructionElement.getAttributeNS(null, Constants._ATT_ALGORITHM);
+        return getLocalAttribute(Constants._ATT_ALGORITHM);
     }
 
     /**
      * Registers implementing class of the SignatureAlgorithm with algorithmURI
      *
      * @param algorithmURI algorithmURI URI representation of <code>SignatureAlgorithm</code>.
-     * @param implementingClass <code>implementingClass</code> the implementing class of 
+     * @param implementingClass <code>implementingClass</code> the implementing class of
      * {@link SignatureAlgorithmSpi}
      * @throws AlgorithmAlreadyRegisteredException if specified algorithmURI is already registered
      * @throws XMLSignatureException
@@ -324,7 +327,7 @@ public class SignatureAlgorithm extends Algorithm {
      */
     @SuppressWarnings("unchecked")
     public static void register(String algorithmURI, String implementingClass)
-       throws AlgorithmAlreadyRegisteredException, ClassNotFoundException, 
+       throws AlgorithmAlreadyRegisteredException, ClassNotFoundException,
            XMLSignatureException {
         JavaUtils.checkRegisterPermission();
         if (log.isDebugEnabled()) {
@@ -340,21 +343,21 @@ public class SignatureAlgorithm extends Algorithm {
             );
         }
         try {
-            Class<? extends SignatureAlgorithmSpi> clazz = 
+            Class<? extends SignatureAlgorithmSpi> clazz =
                 (Class<? extends SignatureAlgorithmSpi>)
                     ClassLoaderUtils.loadClass(implementingClass, SignatureAlgorithm.class);
             algorithmHash.put(algorithmURI, clazz);
         } catch (NullPointerException ex) {
             Object exArgs[] = { algorithmURI, ex.getMessage() };
-            throw new XMLSignatureException("algorithms.NoSuchAlgorithm", exArgs, ex);
+            throw new XMLSignatureException(ex, "algorithms.NoSuchAlgorithm", exArgs);
         }
     }
-    
+
     /**
      * Registers implementing class of the SignatureAlgorithm with algorithmURI
      *
      * @param algorithmURI algorithmURI URI representation of <code>SignatureAlgorithm</code>.
-     * @param implementingClass <code>implementingClass</code> the implementing class of 
+     * @param implementingClass <code>implementingClass</code> the implementing class of
      * {@link SignatureAlgorithmSpi}
      * @throws AlgorithmAlreadyRegisteredException if specified algorithmURI is already registered
      * @throws XMLSignatureException
@@ -362,7 +365,7 @@ public class SignatureAlgorithm extends Algorithm {
      *    caller does not have permission to register the signature algorithm
      */
     public static void register(String algorithmURI, Class<? extends SignatureAlgorithmSpi> implementingClass)
-       throws AlgorithmAlreadyRegisteredException, ClassNotFoundException, 
+       throws AlgorithmAlreadyRegisteredException, ClassNotFoundException,
            XMLSignatureException {
         JavaUtils.checkRegisterPermission();
         if (log.isDebugEnabled()) {
@@ -379,7 +382,7 @@ public class SignatureAlgorithm extends Algorithm {
         }
         algorithmHash.put(algorithmURI, implementingClass);
     }
-    
+
     /**
      * This method registers the default algorithms.
      */
@@ -397,11 +400,11 @@ public class SignatureAlgorithm extends Algorithm {
             XMLSignature.ALGO_ID_MAC_HMAC_SHA1, IntegrityHmac.IntegrityHmacSHA1.class
         );
         algorithmHash.put(
-            XMLSignature.ALGO_ID_SIGNATURE_NOT_RECOMMENDED_RSA_MD5, 
+            XMLSignature.ALGO_ID_SIGNATURE_NOT_RECOMMENDED_RSA_MD5,
             SignatureBaseRSA.SignatureRSAMD5.class
         );
         algorithmHash.put(
-            XMLSignature.ALGO_ID_SIGNATURE_RSA_RIPEMD160, 
+            XMLSignature.ALGO_ID_SIGNATURE_RSA_RIPEMD160,
             SignatureBaseRSA.SignatureRSARIPEMD160.class
         );
         algorithmHash.put(
@@ -416,6 +419,33 @@ public class SignatureAlgorithm extends Algorithm {
         algorithmHash.put(
             XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA512, SignatureBaseRSA.SignatureRSASHA512.class
         );
+        algorithmHash.put(
+            XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1_MGF1, SignatureBaseRSA.SignatureRSASHA1MGF1.class
+        );
+        algorithmHash.put(
+            XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA224_MGF1, SignatureBaseRSA.SignatureRSASHA224MGF1.class
+        );
+        algorithmHash.put(
+            XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA256_MGF1, SignatureBaseRSA.SignatureRSASHA256MGF1.class
+        );
+        algorithmHash.put(
+            XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA384_MGF1, SignatureBaseRSA.SignatureRSASHA384MGF1.class
+        );
+        algorithmHash.put(
+            XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA512_MGF1, SignatureBaseRSA.SignatureRSASHA512MGF1.class
+        );
+        algorithmHash.put(
+            XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_224_MGF1, SignatureBaseRSA.SignatureRSASHA3_224MGF1.class
+        );
+        algorithmHash.put(
+            XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_256_MGF1, SignatureBaseRSA.SignatureRSASHA3_256MGF1.class
+        );
+        algorithmHash.put(
+            XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_384_MGF1, SignatureBaseRSA.SignatureRSASHA3_384MGF1.class
+        );
+        algorithmHash.put(
+            XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_512_MGF1, SignatureBaseRSA.SignatureRSASHA3_512MGF1.class
+        );
         algorithmHash.put(
             XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA1, SignatureECDSA.SignatureECDSASHA1.class
         );
@@ -431,6 +461,9 @@ public class SignatureAlgorithm extends Algorithm {
         algorithmHash.put(
             XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA512, SignatureECDSA.SignatureECDSASHA512.class
         );
+        algorithmHash.put(
+            XMLSignature.ALGO_ID_SIGNATURE_ECDSA_RIPEMD160, SignatureECDSA.SignatureECDSARIPEMD160.class
+        );
         algorithmHash.put(
             XMLSignature.ALGO_ID_MAC_HMAC_NOT_RECOMMENDED_MD5, IntegrityHmac.IntegrityHmacMD5.class
         );
