@@ -73,11 +73,11 @@ import org.w3c.dom.Text;
 public final class XMLSignature extends SignatureElementProxy {
 
     /** MAC - Required HMAC-SHA1 */
-    public static final String ALGO_ID_MAC_HMAC_SHA1 = 
+    public static final String ALGO_ID_MAC_HMAC_SHA1 =
         Constants.SignatureSpecNS + "hmac-sha1";
 
     /** Signature - Required DSAwithSHA1 (DSS) */
-    public static final String ALGO_ID_SIGNATURE_DSA = 
+    public static final String ALGO_ID_SIGNATURE_DSA =
         Constants.SignatureSpecNS + "dsa-sha1";
 
     /** Signature - Optional DSAwithSHA256 */
@@ -85,85 +85,124 @@ public final class XMLSignature extends SignatureElementProxy {
         Constants.SignatureSpec11NS + "dsa-sha256";
 
     /** Signature - Recommended RSAwithSHA1 */
-    public static final String ALGO_ID_SIGNATURE_RSA = 
+    public static final String ALGO_ID_SIGNATURE_RSA =
         Constants.SignatureSpecNS + "rsa-sha1";
-    
+
     /** Signature - Recommended RSAwithSHA1 */
-    public static final String ALGO_ID_SIGNATURE_RSA_SHA1 = 
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA1 =
         Constants.SignatureSpecNS + "rsa-sha1";
-    
+
     /** Signature - NOT Recommended RSAwithMD5 */
-    public static final String ALGO_ID_SIGNATURE_NOT_RECOMMENDED_RSA_MD5 = 
+    public static final String ALGO_ID_SIGNATURE_NOT_RECOMMENDED_RSA_MD5 =
         Constants.MoreAlgorithmsSpecNS + "rsa-md5";
-    
+
     /** Signature - Optional RSAwithRIPEMD160 */
-    public static final String ALGO_ID_SIGNATURE_RSA_RIPEMD160 = 
+    public static final String ALGO_ID_SIGNATURE_RSA_RIPEMD160 =
         Constants.MoreAlgorithmsSpecNS + "rsa-ripemd160";
-    
+
     /** Signature - Optional RSAwithSHA224 */
-    public static final String ALGO_ID_SIGNATURE_RSA_SHA224 = 
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA224 =
         Constants.MoreAlgorithmsSpecNS + "rsa-sha224";
-    
+
     /** Signature - Optional RSAwithSHA256 */
-    public static final String ALGO_ID_SIGNATURE_RSA_SHA256 = 
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA256 =
         Constants.MoreAlgorithmsSpecNS + "rsa-sha256";
-    
+
     /** Signature - Optional RSAwithSHA384 */
-    public static final String ALGO_ID_SIGNATURE_RSA_SHA384 = 
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA384 =
         Constants.MoreAlgorithmsSpecNS + "rsa-sha384";
-    
+
     /** Signature - Optional RSAwithSHA512 */
-    public static final String ALGO_ID_SIGNATURE_RSA_SHA512 = 
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA512 =
         Constants.MoreAlgorithmsSpecNS + "rsa-sha512";
 
+    /** Signature - Optional RSAwithSHA1andMGF1 */
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA1_MGF1 =
+        Constants.XML_DSIG_NS_MORE_07_05 + "sha1-rsa-MGF1";
+
+    /** Signature - Optional RSAwithSHA224andMGF1 */
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA224_MGF1 =
+        Constants.XML_DSIG_NS_MORE_07_05 + "sha224-rsa-MGF1";
+
+    /** Signature - Optional RSAwithSHA256andMGF1 */
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA256_MGF1 =
+        Constants.XML_DSIG_NS_MORE_07_05 + "sha256-rsa-MGF1";
+
+    /** Signature - Optional RSAwithSHA384andMGF1 */
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA384_MGF1 =
+        Constants.XML_DSIG_NS_MORE_07_05 + "sha384-rsa-MGF1";
+
+    /** Signature - Optional RSAwithSHA512andMGF1 */
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA512_MGF1 =
+        Constants.XML_DSIG_NS_MORE_07_05 + "sha512-rsa-MGF1";
+
+    /** Signature - Optional RSAwithSHA3_224andMGF1 */
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA3_224_MGF1 =
+        Constants.XML_DSIG_NS_MORE_07_05 + "sha3-224-rsa-MGF1";
+
+    /** Signature - Optional RSAwithSHA3_256andMGF1 */
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA3_256_MGF1 =
+        Constants.XML_DSIG_NS_MORE_07_05 + "sha3-256-rsa-MGF1";
+
+    /** Signature - Optional RSAwithSHA3_384andMGF1 */
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA3_384_MGF1 =
+        Constants.XML_DSIG_NS_MORE_07_05 + "sha3-384-rsa-MGF1";
+
+    /** Signature - Optional RSAwithSHA3_512andMGF1 */
+    public static final String ALGO_ID_SIGNATURE_RSA_SHA3_512_MGF1 =
+        Constants.XML_DSIG_NS_MORE_07_05 + "sha3-512-rsa-MGF1";
+
     /** HMAC - NOT Recommended HMAC-MD5 */
-    public static final String ALGO_ID_MAC_HMAC_NOT_RECOMMENDED_MD5 = 
+    public static final String ALGO_ID_MAC_HMAC_NOT_RECOMMENDED_MD5 =
         Constants.MoreAlgorithmsSpecNS + "hmac-md5";
-    
+
     /** HMAC - Optional HMAC-RIPEMD160 */
-    public static final String ALGO_ID_MAC_HMAC_RIPEMD160 = 
+    public static final String ALGO_ID_MAC_HMAC_RIPEMD160 =
         Constants.MoreAlgorithmsSpecNS + "hmac-ripemd160";
-    
+
     /** HMAC - Optional HMAC-SHA2224 */
-    public static final String ALGO_ID_MAC_HMAC_SHA224 = 
+    public static final String ALGO_ID_MAC_HMAC_SHA224 =
         Constants.MoreAlgorithmsSpecNS + "hmac-sha224";
-    
+
     /** HMAC - Optional HMAC-SHA256 */
-    public static final String ALGO_ID_MAC_HMAC_SHA256 = 
+    public static final String ALGO_ID_MAC_HMAC_SHA256 =
         Constants.MoreAlgorithmsSpecNS + "hmac-sha256";
-    
+
     /** HMAC - Optional HMAC-SHA284 */
-    public static final String ALGO_ID_MAC_HMAC_SHA384 = 
+    public static final String ALGO_ID_MAC_HMAC_SHA384 =
         Constants.MoreAlgorithmsSpecNS + "hmac-sha384";
-    
+
     /** HMAC - Optional HMAC-SHA512 */
-    public static final String ALGO_ID_MAC_HMAC_SHA512 = 
+    public static final String ALGO_ID_MAC_HMAC_SHA512 =
         Constants.MoreAlgorithmsSpecNS + "hmac-sha512";
-    
+
     /**Signature - Optional ECDSAwithSHA1 */
-    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA1 = 
+    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA1 =
         "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha1";
-    
+
     /**Signature - Optional ECDSAwithSHA224 */
-    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA224 = 
+    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA224 =
         "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha224";
-    
+
     /**Signature - Optional ECDSAwithSHA256 */
-    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA256 = 
+    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA256 =
         "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha256";
-    
+
     /**Signature - Optional ECDSAwithSHA384 */
-    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA384 = 
+    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA384 =
         "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha384";
-    
+
     /**Signature - Optional ECDSAwithSHA512 */
-    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA512 = 
+    public static final String ALGO_ID_SIGNATURE_ECDSA_SHA512 =
         "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha512";
 
-    /** {@link org.apache.commons.logging} logging facility */
-    private static org.apache.commons.logging.Log log = 
-        org.apache.commons.logging.LogFactory.getLog(XMLSignature.class);
-    
+    /**Signature - Optional ECDSAwithRIPEMD160 */
+    public static final String ALGO_ID_SIGNATURE_ECDSA_RIPEMD160 =
+        "http://www.w3.org/2007/05/xmldsig-more#ecdsa-ripemd160";
+
+    private static org.slf4j.Logger log =
+        org.slf4j.LoggerFactory.getLogger(XMLSignature.class);
+
     /** ds:Signature.ds:SignedInfo element */
     private SignedInfo signedInfo;
 
@@ -212,7 +251,7 @@ public final class XMLSignature extends SignatureElementProxy {
     public XMLSignature(Document doc, String baseURI, String signatureMethodURI,
                         int hmacOutputLength) throws XMLSecurityException {
         this(
-            doc, baseURI, signatureMethodURI, hmacOutputLength, 
+            doc, baseURI, signatureMethodURI, hmacOutputLength,
             Canonicalizer.ALGO_ID_C14N_OMIT_COMMENTS
         );
     }
@@ -223,13 +262,13 @@ public final class XMLSignature extends SignatureElementProxy {
      * @param doc
      * @param baseURI
      * @param signatureMethodURI the Signature method to be used.
-     * @param canonicalizationMethodURI the canonicalization algorithm to be 
+     * @param canonicalizationMethodURI the canonicalization algorithm to be
      * used to c14nize the SignedInfo element.
      * @throws XMLSecurityException
      */
     public XMLSignature(
-        Document doc, 
-        String baseURI, 
+        Document doc,
+        String baseURI,
         String signatureMethodURI,
         String canonicalizationMethodURI
     ) throws XMLSecurityException {
@@ -247,41 +286,41 @@ public final class XMLSignature extends SignatureElementProxy {
      * @throws XMLSecurityException
      */
     public XMLSignature(
-        Document doc, 
-        String baseURI, 
+        Document doc,
+        String baseURI,
         String signatureMethodURI,
-        int hmacOutputLength, 
+        int hmacOutputLength,
         String canonicalizationMethodURI
     ) throws XMLSecurityException {
         super(doc);
 
         String xmlnsDsPrefix = getDefaultPrefix(Constants.SignatureSpecNS);
         if (xmlnsDsPrefix == null || xmlnsDsPrefix.length() == 0) {
-            this.constructionElement.setAttributeNS(
+            getElement().setAttributeNS(
                 Constants.NamespaceSpecNS, "xmlns", Constants.SignatureSpecNS
             );
         } else {
-            this.constructionElement.setAttributeNS(
+            getElement().setAttributeNS(
                 Constants.NamespaceSpecNS, "xmlns:" + xmlnsDsPrefix, Constants.SignatureSpecNS
             );
         }
-        XMLUtils.addReturnToElement(this.constructionElement);
+        addReturnToSelf();
 
         this.baseURI = baseURI;
-        this.signedInfo = 
+        this.signedInfo =
             new SignedInfo(
-                this.doc, signatureMethodURI, hmacOutputLength, canonicalizationMethodURI
+                getDocument(), signatureMethodURI, hmacOutputLength, canonicalizationMethodURI
             );
 
-        this.constructionElement.appendChild(this.signedInfo.getElement());
-        XMLUtils.addReturnToElement(this.constructionElement);
+        appendSelf(this.signedInfo);
+        addReturnToSelf();
 
         // create an empty SignatureValue; this is filled by setSignatureValueElement
-        signatureValueElement = 
-            XMLUtils.createElementInSignatureSpace(this.doc, Constants._TAG_SIGNATUREVALUE);
+        signatureValueElement =
+            XMLUtils.createElementInSignatureSpace(getDocument(), Constants._TAG_SIGNATUREVALUE);
 
-        this.constructionElement.appendChild(signatureValueElement);
-        XMLUtils.addReturnToElement(this.constructionElement);
+        appendSelf(signatureValueElement);
+        addReturnToSelf();
     }
 
     /**
@@ -293,40 +332,40 @@ public final class XMLSignature extends SignatureElementProxy {
      * @throws XMLSecurityException
      */
     public XMLSignature(
-        Document doc, 
-        String baseURI, 
-        Element SignatureMethodElem, 
+        Document doc,
+        String baseURI,
+        Element SignatureMethodElem,
         Element CanonicalizationMethodElem
     ) throws XMLSecurityException {
         super(doc);
 
         String xmlnsDsPrefix = getDefaultPrefix(Constants.SignatureSpecNS);
         if (xmlnsDsPrefix == null || xmlnsDsPrefix.length() == 0) {
-            this.constructionElement.setAttributeNS(
+            getElement().setAttributeNS(
                 Constants.NamespaceSpecNS, "xmlns", Constants.SignatureSpecNS
             );
         } else {
-            this.constructionElement.setAttributeNS(
+            getElement().setAttributeNS(
                 Constants.NamespaceSpecNS, "xmlns:" + xmlnsDsPrefix, Constants.SignatureSpecNS
             );
         }
-        XMLUtils.addReturnToElement(this.constructionElement);
+        addReturnToSelf();
 
         this.baseURI = baseURI;
-        this.signedInfo = 
-            new SignedInfo(this.doc, SignatureMethodElem, CanonicalizationMethodElem);
+        this.signedInfo =
+            new SignedInfo(getDocument(), SignatureMethodElem, CanonicalizationMethodElem);
 
-        this.constructionElement.appendChild(this.signedInfo.getElement());
-        XMLUtils.addReturnToElement(this.constructionElement);
+        appendSelf(this.signedInfo);
+        addReturnToSelf();
 
         // create an empty SignatureValue; this is filled by setSignatureValueElement
-        signatureValueElement = 
-            XMLUtils.createElementInSignatureSpace(this.doc, Constants._TAG_SIGNATUREVALUE);
+        signatureValueElement =
+            XMLUtils.createElementInSignatureSpace(getDocument(), Constants._TAG_SIGNATUREVALUE);
 
-        this.constructionElement.appendChild(signatureValueElement);
-        XMLUtils.addReturnToElement(this.constructionElement);
+        appendSelf(signatureValueElement);
+        addReturnToSelf();
     }
-    
+
     /**
      * This will parse the element and construct the Java Objects.
      * That will allow a user to validate the signature.
@@ -370,7 +409,7 @@ public final class XMLSignature extends SignatureElementProxy {
         signedInfoElem = XMLUtils.getNextElement(element.getFirstChild());
 
         // check out SignatureValue child
-        this.signatureValueElement = 
+        this.signatureValueElement =
             XMLUtils.getNextElement(signedInfoElem.getNextSibling());
 
         // check to see if it exists
@@ -384,17 +423,17 @@ public final class XMLSignature extends SignatureElementProxy {
         }
 
         // <element ref="ds:KeyInfo" minOccurs="0"/>
-        Element keyInfoElem = 
+        Element keyInfoElem =
             XMLUtils.getNextElement(signatureValueElement.getNextSibling());
 
         // If it exists use it, but it's not mandatory
-        if (keyInfoElem != null 
-            && Constants.SignatureSpecNS.equals(keyInfoElem.getNamespaceURI()) 
+        if (keyInfoElem != null
+            && Constants.SignatureSpecNS.equals(keyInfoElem.getNamespaceURI())
             && Constants._TAG_KEYINFO.equals(keyInfoElem.getLocalName())) {
             this.keyInfo = new KeyInfo(keyInfoElem, baseURI);
             this.keyInfo.setSecureValidation(secureValidation);
         }
-        
+
         // <element ref="ds:Object" minOccurs="0" maxOccurs="unbounded"/>
         Element objectElem =
             XMLUtils.getNextElement(signatureValueElement.getNextSibling());
@@ -421,7 +460,7 @@ public final class XMLSignature extends SignatureElementProxy {
 
             objectElem = XMLUtils.getNextElement(objectElem.getNextSibling());
         }
-        
+
         this.state = MODE_VERIFY;
     }
 
@@ -432,8 +471,7 @@ public final class XMLSignature extends SignatureElementProxy {
      */
     public void setId(String id) {
         if (id != null) {
-            this.constructionElement.setAttributeNS(null, Constants._ATT_ID, id);
-            this.constructionElement.setIdAttributeNS(null, Constants._ATT_ID, true);
+            setLocalIdAttribute(Constants._ATT_ID, id);
         }
     }
 
@@ -443,7 +481,7 @@ public final class XMLSignature extends SignatureElementProxy {
      * @return the <code>Id</code> attribute
      */
     public String getId() {
-        return this.constructionElement.getAttributeNS(null, Constants._ATT_ID);
+        return getLocalAttribute(Constants._ATT_ID);
     }
 
     /**
@@ -466,7 +504,7 @@ public final class XMLSignature extends SignatureElementProxy {
         try {
             return Base64.decode(signatureValueElement);
         } catch (Base64DecodingException ex) {
-            throw new XMLSignatureException("empty", ex);
+            throw new XMLSignatureException(ex, "empty");
         }
     }
 
@@ -488,7 +526,7 @@ public final class XMLSignature extends SignatureElementProxy {
             base64codedValue = "\n" + base64codedValue + "\n";
         }
 
-        Text t = this.doc.createTextNode(base64codedValue);
+        Text t = createText(base64codedValue);
         signatureValueElement.appendChild(t);
     }
 
@@ -505,24 +543,24 @@ public final class XMLSignature extends SignatureElementProxy {
         if (this.state == MODE_SIGN && this.keyInfo == null) {
 
             // create the KeyInfo
-            this.keyInfo = new KeyInfo(this.doc);
+            this.keyInfo = new KeyInfo(getDocument());
 
             // get the Element from KeyInfo
             Element keyInfoElement = this.keyInfo.getElement();
-            Element firstObject = 
+            Element firstObject =
                 XMLUtils.selectDsNode(
-                    this.constructionElement.getFirstChild(), Constants._TAG_OBJECT, 0
+                    getElement().getFirstChild(), Constants._TAG_OBJECT, 0
                 );
 
             if (firstObject != null) {
                 // add it before the object
-                this.constructionElement.insertBefore(keyInfoElement, firstObject);
-                XMLUtils.addReturnBeforeChild(this.constructionElement, firstObject);
+                getElement().insertBefore(keyInfoElement, firstObject);
+                XMLUtils.addReturnBeforeChild(getElement(), firstObject);
             } else {
                 // add it as the last element to the signature
-                this.constructionElement.appendChild(keyInfoElement);
-                XMLUtils.addReturnToElement(this.constructionElement);
-            }         
+                appendSelf(keyInfoElement);
+                addReturnToSelf();
+            }
         }
 
         return this.keyInfo;
@@ -543,10 +581,10 @@ public final class XMLSignature extends SignatureElementProxy {
         //  "signature.operationOnlyBeforeSign");
         //}
 
-        this.constructionElement.appendChild(object.getElement());
-        XMLUtils.addReturnToElement(this.constructionElement);
+        appendSelf(object);
+        addReturnToSelf();
         //} catch (XMLSecurityException ex) {
-        // throw new XMLSignatureException("empty", ex);
+        // throw new XMLSignatureException(ex);
         //}
     }
 
@@ -559,9 +597,9 @@ public final class XMLSignature extends SignatureElementProxy {
      * or null if no such <code>ds:Object</code> element exists.
      */
     public ObjectContainer getObjectItem(int i) {
-        Element objElem = 
+        Element objElem =
             XMLUtils.selectDsNode(
-                this.constructionElement.getFirstChild(), Constants._TAG_OBJECT, i
+                getFirstChild(), Constants._TAG_OBJECT, i
             );
 
         try {
@@ -581,10 +619,10 @@ public final class XMLSignature extends SignatureElementProxy {
     }
 
     /**
-     * Digests all References in the SignedInfo, calculates the signature value 
+     * Digests all References in the SignedInfo, calculates the signature value
      * and sets it in the SignatureValue Element.
      *
-     * @param signingKey the {@link java.security.PrivateKey} or 
+     * @param signingKey the {@link java.security.PrivateKey} or
      * {@link javax.crypto.SecretKey} that is used to sign.
      * @throws XMLSignatureException
      */
@@ -602,11 +640,12 @@ public final class XMLSignature extends SignatureElementProxy {
             SignatureAlgorithm sa = si.getSignatureAlgorithm();
             OutputStream so = null;
             try {
-                // initialize SignatureAlgorithm for signing
-                sa.initSign(signingKey);            
-
                 // generate digest values for all References in this SignedInfo
                 si.generateDigestValues();
+
+                // initialize SignatureAlgorithm for signing
+                sa.initSign(signingKey);
+
                 so = new UnsyncBufferedOutputStream(new SignerOutputStream(sa));
                 // get the canonicalized bytes from SignedInfo
                 si.signInOctetStream(so);
@@ -618,7 +657,7 @@ public final class XMLSignature extends SignatureElementProxy {
                         so.close();
                     } catch (IOException ex) {
                         if (log.isDebugEnabled()) {
-                            log.debug(ex);
+                            log.debug(ex.getMessage(), ex);
                         }
                     }
                 }
@@ -629,11 +668,11 @@ public final class XMLSignature extends SignatureElementProxy {
         } catch (XMLSignatureException ex) {
             throw ex;
         } catch (CanonicalizationException ex) {
-            throw new XMLSignatureException("empty", ex);
+            throw new XMLSignatureException(ex);
         } catch (InvalidCanonicalizerException ex) {
-            throw new XMLSignatureException("empty", ex);
+            throw new XMLSignatureException(ex);
         } catch (XMLSecurityException ex) {
-            throw new XMLSignatureException("empty", ex);
+            throw new XMLSignatureException(ex);
         }
     }
 
@@ -661,7 +700,7 @@ public final class XMLSignature extends SignatureElementProxy {
      * stored DigestValues and then checking to see if the Signatures match on
      * the SignedInfo.
      *
-     * @param cert Certificate that contains the public key part of the keypair 
+     * @param cert Certificate that contains the public key part of the keypair
      * that was used to sign.
      * @return true if the signature is valid, false otherwise
      * @throws XMLSignatureException
@@ -672,7 +711,7 @@ public final class XMLSignature extends SignatureElementProxy {
         if (cert != null) {
             // check the values with the public key from the cert
             return this.checkSignatureValue(cert.getPublicKey());
-        } 
+        }
 
         Object exArgs[] = { "Didn't get a certificate" };
         throw new XMLSignatureException("empty", exArgs);
@@ -683,7 +722,7 @@ public final class XMLSignature extends SignatureElementProxy {
      * comparing those against the stored DigestValues and then checking to see
      * if the Signatures match on the SignedInfo.
      *
-     * @param pk {@link java.security.PublicKey} part of the keypair or 
+     * @param pk {@link java.security.PublicKey} part of the keypair or
      * {@link javax.crypto.SecretKey} that was used to sign
      * @return true if the signature is valid, false otherwise
      * @throws XMLSignatureException
@@ -704,7 +743,7 @@ public final class XMLSignature extends SignatureElementProxy {
             SignedInfo si = this.getSignedInfo();
             //create a SignatureAlgorithms from the SignatureMethod inside
             //SignedInfo. This is used to validate the signature.
-            SignatureAlgorithm sa = si.getSignatureAlgorithm();               
+            SignatureAlgorithm sa = si.getSignatureAlgorithm();
             if (log.isDebugEnabled()) {
                 log.debug("signatureMethodURI = " + sa.getAlgorithmURI());
                 log.debug("jceSigAlgorithm    = " + sa.getJCEAlgorithmString());
@@ -725,14 +764,14 @@ public final class XMLSignature extends SignatureElementProxy {
                 sigBytes = this.getSignatureValue();
             } catch (IOException ex) {
                 if (log.isDebugEnabled()) {
-                    log.debug(ex);
+                    log.debug(ex.getMessage(), ex);
                 }
                 // Impossible...
             } catch (XMLSecurityException ex) {
                 throw ex;
             }
 
-            // have SignatureAlgorithm sign the input bytes and compare them to 
+            // have SignatureAlgorithm sign the input bytes and compare them to
             // the bytes that were stored in the signature.
             if (!sa.verify(sigBytes)) {
                 log.warn("Signature verification failed.");
@@ -743,16 +782,16 @@ public final class XMLSignature extends SignatureElementProxy {
         } catch (XMLSignatureException ex) {
             throw ex;
         } catch (XMLSecurityException ex) {
-            throw new XMLSignatureException("empty", ex);
-        } 
+            throw new XMLSignatureException(ex);
+        }
     }
 
     /**
      * Add a Reference with full parameters to this Signature
      *
-     * @param referenceURI URI of the resource to be signed. Can be null in 
-     * which case the dereferencing is application specific. Can be "" in which 
-     * it's the parent node (or parent document?). There can only be one "" in 
+     * @param referenceURI URI of the resource to be signed. Can be null in
+     * which case the dereferencing is application specific. Can be "" in which
+     * it's the parent node (or parent document?). There can only be one "" in
      * each signature.
      * @param trans Optional list of transformations to be done before digesting
      * @param digestURI Mandatory URI of the digesting algorithm to use.
@@ -761,10 +800,10 @@ public final class XMLSignature extends SignatureElementProxy {
      * @throws XMLSignatureException
      */
     public void addDocument(
-        String referenceURI, 
-        Transforms trans, 
-        String digestURI, 
-        String referenceId, 
+        String referenceURI,
+        Transforms trans,
+        String digestURI,
+        String referenceId,
         String referenceType
     ) throws XMLSignatureException {
         this.signedInfo.addDocument(
@@ -782,8 +821,8 @@ public final class XMLSignature extends SignatureElementProxy {
      * @throws XMLSignatureException
      */
     public void addDocument(
-        String referenceURI, 
-        Transforms trans, 
+        String referenceURI,
+        Transforms trans,
         String digestURI
     ) throws XMLSignatureException {
         this.signedInfo.addDocument(this.baseURI, referenceURI, trans, digestURI, null, null);
@@ -826,7 +865,7 @@ public final class XMLSignature extends SignatureElementProxy {
      * @throws XMLSecurityException
      */
     public void addKeyInfo(X509Certificate cert) throws XMLSecurityException {
-        X509Data x509data = new X509Data(this.doc);
+        X509Data x509data = new X509Data(getDocument());
 
         x509data.addCertificate(cert);
         this.getKeyInfo().add(x509data);
@@ -843,8 +882,8 @@ public final class XMLSignature extends SignatureElementProxy {
     }
 
     /**
-     * Proxy method for {@link SignedInfo#createSecretKey(byte[])}. If you want 
-     * to create a MAC, this method helps you to obtain the 
+     * Proxy method for {@link SignedInfo#createSecretKey(byte[])}. If you want
+     * to create a MAC, this method helps you to obtain the
      * {@link javax.crypto.SecretKey} from octets.
      *
      * @param secretKeyBytes
