@@ -32,7 +32,7 @@ import org.w3c.dom.Text;
 
 /**
  * Implementation of MIME's Base64 encoding and decoding conversions.
- * Optimized code. (raw version taken from oreilly.jonathan.util, 
+ * Optimized code. (raw version taken from oreilly.jonathan.util,
  * and currently org.apache.xerces.ds.util.Base64)
  *
  * @author Raul Benito(Of the xerces copy, and little adaptations).
@@ -45,7 +45,7 @@ public class Base64 {
 
     /** Field BASE64DEFAULTLENGTH */
     public static final int BASE64DEFAULTLENGTH = 76;
-    
+
     private static final int BASELENGTH = 255;
     private static final int LOOKUPLENGTH = 64;
     private static final int TWENTYFOURBITGROUP = 24;
@@ -53,7 +53,7 @@ public class Base64 {
     private static final int SIXTEENBIT = 16;
     private static final int FOURBYTE = 4;
     private static final int SIGN = -128;
-    private static final char PAD = '=';   
+    private static final char PAD = '=';
     private static final byte [] base64Alphabet = new byte[BASELENGTH];
     private static final char [] lookUpBase64Alphabet = new char[LOOKUPLENGTH];
 
@@ -116,8 +116,8 @@ public class Base64 {
 
         byte[] bigBytes = big.toByteArray();
 
-        if (((big.bitLength() % 8) != 0)
-            && (((big.bitLength() / 8) + 1) == (bitlen / 8))) {
+        if (big.bitLength() % 8 != 0
+            && big.bitLength() / 8 + 1 == bitlen / 8) {
             return bigBytes;
         }
 
@@ -125,7 +125,7 @@ public class Base64 {
         int startSrc = 0;    // no need to skip anything
         int bigLen = bigBytes.length;    //valid length of the string
 
-        if ((big.bitLength() % 8) == 0) {    // correct values
+        if (big.bitLength() % 8 == 0) {    // correct values
             startSrc = 1;    // skip sign bit
 
             bigLen--;    // valid length of the string
@@ -160,7 +160,7 @@ public class Base64 {
      * @param bitlen <code>int<code> the desired length in bits of the representation
      * @return a byte array with <code>bitlen</code> bits of <code>big</code>
      */
-    public static final  byte[] encode(BigInteger big, int bitlen) {
+    public static final byte[] encode(BigInteger big, int bitlen) {
 
         //round bitlen
         bitlen = ((bitlen + 7) >> 3) << 3;
@@ -171,8 +171,8 @@ public class Base64 {
 
         byte[] bigBytes = big.toByteArray();
 
-        if (((big.bitLength() % 8) != 0)
-            && (((big.bitLength() / 8) + 1) == (bitlen / 8))) {
+        if (big.bitLength() % 8 != 0
+            && big.bitLength() / 8 + 1 == bitlen / 8) {
             return bigBytes;
         }
 
@@ -180,7 +180,7 @@ public class Base64 {
         int startSrc = 0;    // no need to skip anything
         int bigLen = bigBytes.length;    //valid length of the string
 
-        if ((big.bitLength() % 8) == 0) {    // correct values
+        if (big.bitLength() % 8 == 0) {    // correct values
             startSrc = 1;    // skip sign bit
 
             bigLen--;    // valid length of the string
@@ -201,21 +201,20 @@ public class Base64 {
      * @return the biginteger obtained from the node
      * @throws Base64DecodingException
      */
-    public static final BigInteger decodeBigIntegerFromElement(Element element) 
+    public static final BigInteger decodeBigIntegerFromElement(Element element)
         throws Base64DecodingException {
         return new BigInteger(1, Base64.decode(element));
     }
 
     /**
-     * Method decodeBigIntegerFromText
-     *
-     * @param text
-     * @return the biginter obtained from the text node
+     * Decode a base 64 string into a {@link BigInteger}
+     * @param base64str Base 64 encoded string.
+     * @return a decoded BigInteger
      * @throws Base64DecodingException
      */
-    public static final BigInteger decodeBigIntegerFromText(Text text) 
-        throws Base64DecodingException {
-        return new BigInteger(1, Base64.decode(text.getData()));
+    public static BigInteger decodeBigIntegerFromString(String base64str)
+            throws Base64DecodingException {
+        return new BigInteger(1, Base64.decode(base64str));
     }
 
     /**
@@ -253,7 +252,7 @@ public class Base64 {
     public static final byte[] decode(Element element) throws Base64DecodingException {
 
         Node sibling = element.getFirstChild();
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
         while (sibling != null) {
             if (sibling.getNodeType() == Node.TEXT_NODE) {
@@ -293,12 +292,12 @@ public class Base64 {
      * @throws Base64DecodingException
      *
      */
-    public static final byte[] decode(byte[] base64) throws Base64DecodingException  {   	   
+    public static final byte[] decode(byte[] base64) throws Base64DecodingException  {   	
         return decodeInternal(base64, -1);
     }
 
     /**
-     * Encode a byte array and fold lines at the standard 76th character unless 
+     * Encode a byte array and fold lines at the standard 76th character unless
      * ignore line breaks property is set.
      *
      * @param binaryData <code>byte[]<code> to be base64 encoded
@@ -306,7 +305,7 @@ public class Base64 {
      */
     public static final String encode(byte[] binaryData) {
         return XMLUtils.ignoreLineBreaks()
-            ? encode(binaryData, Integer.MAX_VALUE) 
+            ? encode(binaryData, Integer.MAX_VALUE)
             : encode(binaryData, BASE64DEFAULTLENGTH);
     }
 
@@ -324,11 +323,9 @@ public class Base64 {
         throws IOException, Base64DecodingException {
 
         byte[] retBytes = null;
-        UnsyncByteArrayOutputStream baos = null;
+        UnsyncByteArrayOutputStream baos = new UnsyncByteArrayOutputStream();
         try {
-            baos = new UnsyncByteArrayOutputStream();
             String line;
-    
             while (null != (line = reader.readLine())) {
                 byte[] bytes = decode(line);
                 baos.write(bytes);
@@ -341,12 +338,12 @@ public class Base64 {
         return retBytes;
     }
 
-    protected static final boolean isWhiteSpace(byte octect) {
-        return (octect == 0x20 || octect == 0xd || octect == 0xa || octect == 0x9);
+    protected static final boolean isWhiteSpace(byte octet) {
+        return octet == 0x20 || octet == 0xd || octet == 0xa || octet == 0x9;
     }
 
-    protected static final boolean isPad(byte octect) {
-        return (octect == PAD);
+    protected static final boolean isPad(byte octet) {
+        return octet == PAD;
     }
 
     /**
@@ -363,7 +360,7 @@ public class Base64 {
      * @param length <code>int<code> length of wrapped lines; No wrapping if less than 4.
      * @return a <code>String</code> with encoded data
      */
-    public static final String  encode(byte[] binaryData,int length) {
+    public static final String  encode(byte[] binaryData, int length) {
         if (length < 4) {
             length = Integer.MAX_VALUE;
         }
@@ -372,19 +369,19 @@ public class Base64 {
             return null;
         }
 
-        int lengthDataBits = binaryData.length * EIGHTBIT;
-        if (lengthDataBits == 0) {
+        long lengthDataBits = ((long) binaryData.length) * ((long) EIGHTBIT);
+        if (lengthDataBits == 0L) {
             return "";
         }
 
-        int fewerThan24bits = lengthDataBits % TWENTYFOURBITGROUP;
-        int numberTriplets = lengthDataBits / TWENTYFOURBITGROUP;
-        int numberQuartet = fewerThan24bits != 0 ? numberTriplets + 1 : numberTriplets;
+        long fewerThan24bits = lengthDataBits % ((long) TWENTYFOURBITGROUP);
+        int numberTriplets = (int) (lengthDataBits / TWENTYFOURBITGROUP);
+        int numberQuartet = fewerThan24bits != 0L ? numberTriplets + 1 : numberTriplets;
         int quartesPerLine = length / 4;
         int numberLines = (numberQuartet - 1) / quartesPerLine;
         char encodedData[] = null;
 
-        encodedData = new char[numberQuartet * 4 + numberLines];
+        encodedData = new char[numberQuartet * 4 + numberLines * 2];
 
         byte k = 0, l = 0, b1 = 0, b2 = 0, b3 = 0;
         int encodedIndex = 0;
@@ -412,8 +409,9 @@ public class Base64 {
                 encodedData[encodedIndex++] = lookUpBase64Alphabet[b3 & 0x3f];
 
                 i++;
-            }           
-            encodedData[encodedIndex++] = 0xa;           
+            }
+            encodedData[encodedIndex++] = 0xd;
+            encodedData[encodedIndex++] = 0xa;
         }
 
         for (; i < numberTriplets; i++) {
@@ -428,8 +426,8 @@ public class Base64 {
 
             byte val2 = ((b2 & SIGN) == 0) ? (byte)(b2 >> 4) : (byte)((b2) >> 4 ^ 0xf0);
             byte val3 = ((b3 & SIGN) == 0) ? (byte)(b3 >> 6) : (byte)((b3) >> 6 ^ 0xfc);
-    
-    
+
+
             encodedData[encodedIndex++] = lookUpBase64Alphabet[val1];
             encodedData[encodedIndex++] = lookUpBase64Alphabet[val2 | (k << 4)];
             encodedData[encodedIndex++] = lookUpBase64Alphabet[(l << 2) | val3];
@@ -453,7 +451,7 @@ public class Base64 {
 
             byte val1 = ((b1 & SIGN) == 0) ? (byte)(b1 >> 2) : (byte)((b1) >> 2 ^ 0xc0);
             byte val2 = ((b2 & SIGN) == 0) ? (byte)(b2 >> 4) : (byte)((b2) >> 4 ^ 0xf0);
-    
+
             encodedData[encodedIndex++] = lookUpBase64Alphabet[val1];
             encodedData[encodedIndex++] = lookUpBase64Alphabet[val2 | (k << 4)];
             encodedData[encodedIndex++] = lookUpBase64Alphabet[l << 2];
@@ -494,7 +492,7 @@ public class Base64 {
         return newSize;
     }
 
-    protected static final byte[] decodeInternal(byte[] base64Data, int len) 
+    protected static final byte[] decodeInternal(byte[] base64Data, int len)
         throws Base64DecodingException {
         // remove white spaces
         if (len == -1) {
@@ -506,7 +504,7 @@ public class Base64 {
             //should be divisible by four
         }
 
-        int numberQuadruple = (len / FOURBYTE);
+        int numberQuadruple = len / FOURBYTE;
 
         if (numberQuadruple == 0) {
             return new byte[0];
@@ -525,7 +523,7 @@ public class Base64 {
         //first last bits.
         b1 = base64Alphabet[base64Data[dataIndex++]];
         b2 = base64Alphabet[base64Data[dataIndex++]];
-        if ((b1==-1) || (b2==-1)) {
+        if (b1 == -1 || b2 == -1) {
              //if found "no data" just return null
             throw new Base64DecodingException("decoding.general");
         }
@@ -534,21 +532,21 @@ public class Base64 {
         byte d3, d4;
         b3 = base64Alphabet[d3 = base64Data[dataIndex++]];
         b4 = base64Alphabet[d4 = base64Data[dataIndex++]];
-        if ((b3 == -1) || (b4 == -1) ) {
+        if (b3 == -1 || b4 == -1) {
             //Check if they are PAD characters
             if (isPad(d3) && isPad(d4)) {               //Two PAD e.g. 3c[Pad][Pad]
                 if ((b2 & 0xf) != 0) { //last 4 bits should be zero
                     throw new Base64DecodingException("decoding.general");
                 }
-                decodedData = new byte[encodedIndex + 1];                
-                decodedData[encodedIndex]   = (byte)(b1 << 2 | b2 >> 4) ;                
-            } else if (!isPad(d3) && isPad(d4)) {               //One PAD  e.g. 3cQ[Pad]                
+                decodedData = new byte[encodedIndex + 1];
+                decodedData[encodedIndex]   = (byte)(b1 << 2 | b2 >> 4) ;
+            } else if (!isPad(d3) && isPad(d4)) {               //One PAD  e.g. 3cQ[Pad]
                 if ((b3 & 0x3) != 0) { //last 2 bits should be zero
                     throw new Base64DecodingException("decoding.general");
                 }
-                decodedData = new byte[encodedIndex + 2];                
+                decodedData = new byte[encodedIndex + 2];
                 decodedData[encodedIndex++] = (byte)(b1 << 2 | b2 >> 4);
-                decodedData[encodedIndex] = (byte)(((b2 & 0xf) << 4) |((b3 >> 2) & 0xf));                
+                decodedData[encodedIndex] = (byte)(((b2 & 0xf) << 4) |((b3 >> 2) & 0xf));
             } else {
                 //an error  like "3c[Pad]r", "3cdX", "3cXd", "3cXX" where X is non data
                 throw new Base64DecodingException("decoding.general");
@@ -569,18 +567,15 @@ public class Base64 {
             b3 = base64Alphabet[base64Data[dataIndex++]];
             b4 = base64Alphabet[base64Data[dataIndex++]];
 
-            if ((b1 == -1) ||
-                (b2 == -1) ||
-                (b3 == -1) ||
-                (b4 == -1)) {
-                //if found "no data" just return null   
+            if (b1 == -1 || b2 == -1 || b3 == -1 || b4 == -1) {
+                //if found "no data" just return null
                 throw new Base64DecodingException("decoding.general");
             }
 
             decodedData[encodedIndex++] = (byte)(b1 << 2 | b2 >> 4) ;
             decodedData[encodedIndex++] = (byte)(((b2 & 0xf) << 4) |((b3 >> 2) & 0xf));
             decodedData[encodedIndex++] = (byte)(b3 << 6 | b4 );
-        }            
+        }
         return decodedData;
     }
 
@@ -592,11 +587,11 @@ public class Base64 {
      * @throws IOException
      * @throws Base64DecodingException
      */
-    public static final void decode(String base64Data, OutputStream os) 
+    public static final void decode(String base64Data, OutputStream os)
         throws Base64DecodingException, IOException {
         byte[] bytes = new byte[base64Data.length()];
         int len = getBytesInternal(base64Data, bytes);
-        decode(bytes,os,len);
+        decode(bytes, os, len);
     }
 
     /**
@@ -607,13 +602,13 @@ public class Base64 {
      * @throws IOException
      * @throws Base64DecodingException
      */
-    public static final void decode(byte[] base64Data, OutputStream os) 
-        throws Base64DecodingException, IOException {	    
-        decode(base64Data,os,-1);
+    public static final void decode(byte[] base64Data, OutputStream os)
+        throws Base64DecodingException, IOException {	
+        decode(base64Data, os, -1);
     }
 
-    protected static final void decode(byte[] base64Data, OutputStream os, int len) 
-        throws Base64DecodingException, IOException {	    
+    protected static final void decode(byte[] base64Data, OutputStream os, int len)
+        throws Base64DecodingException, IOException {	
         // remove white spaces
         if (len == -1) {
             len = removeWhiteSpace(base64Data);
@@ -624,7 +619,7 @@ public class Base64 {
             //should be divisible by four
         }
 
-        int numberQuadruple = (len / FOURBYTE);
+        int numberQuadruple = len / FOURBYTE;
 
         if (numberQuadruple == 0) {
             return;
@@ -634,31 +629,28 @@ public class Base64 {
         byte b1 = 0, b2 = 0, b3 = 0, b4 = 0;
 
         int i = 0;
-        int dataIndex = 0;    
+        int dataIndex = 0;
 
         //the begin
-        for (i=numberQuadruple - 1; i > 0; i--) {
+        for (i = numberQuadruple - 1; i > 0; i--) {
             b1 = base64Alphabet[base64Data[dataIndex++]];
             b2 = base64Alphabet[base64Data[dataIndex++]];
             b3 = base64Alphabet[base64Data[dataIndex++]];
             b4 = base64Alphabet[base64Data[dataIndex++]];
-            if ((b1 == -1) ||
-                (b2 == -1) ||
-                (b3 == -1) ||
-                (b4 == -1) ) {
+            if (b1 == -1 || b2 == -1 || b3 == -1 || b4 == -1) {
                 //if found "no data" just return null
                 throw new Base64DecodingException("decoding.general");
             }
 
             os.write((byte)(b1 << 2 | b2 >> 4));
             os.write((byte)(((b2 & 0xf) << 4 ) | ((b3 >> 2) & 0xf)));
-            os.write( (byte)(b3 << 6 | b4));
-        }   
+            os.write((byte)(b3 << 6 | b4));
+        }
         b1 = base64Alphabet[base64Data[dataIndex++]];
         b2 = base64Alphabet[base64Data[dataIndex++]];
 
         //  first last bits.
-        if ((b1 == -1) || (b2 == -1) ) {
+        if (b1 == -1 || b2 == -1) {
             //if found "no data" just return null
             throw new Base64DecodingException("decoding.general");
         }
@@ -666,24 +658,24 @@ public class Base64 {
         byte d3, d4;
         b3 = base64Alphabet[d3 = base64Data[dataIndex++]];
         b4 = base64Alphabet[d4 = base64Data[dataIndex++]];
-        if ((b3 == -1 ) || (b4 == -1) ) { //Check if they are PAD characters
+        if (b3 == -1 || b4 == -1) { //Check if they are PAD characters
             if (isPad(d3) && isPad(d4)) {               //Two PAD e.g. 3c[Pad][Pad]
                 if ((b2 & 0xf) != 0) { //last 4 bits should be zero
                     throw new Base64DecodingException("decoding.general");
                 }
-                os.write((byte)(b1 << 2 | b2 >> 4));                
-            } else if (!isPad(d3) && isPad(d4)) {               //One PAD  e.g. 3cQ[Pad]             
+                os.write((byte)(b1 << 2 | b2 >> 4));
+            } else if (!isPad(d3) && isPad(d4)) {               //One PAD  e.g. 3cQ[Pad]
                 if ((b3 & 0x3 ) != 0) { //last 2 bits should be zero
                     throw new Base64DecodingException("decoding.general");
                 }
                 os.write((byte)(b1 << 2 | b2 >> 4));
-                os.write((byte)(((b2 & 0xf) << 4) | ((b3 >> 2) & 0xf)));                
+                os.write((byte)(((b2 & 0xf) << 4) | ((b3 >> 2) & 0xf)));
             } else {
                 //an error  like "3c[Pad]r", "3cdX", "3cXd", "3cXX" where X is non data
                 throw new Base64DecodingException("decoding.general");
             }
         } else {
-            //No PAD e.g 3cQl         
+            //No PAD e.g 3cQl
             os.write((byte)(b1 << 2 | b2 >> 4));
             os.write( (byte)(((b2 & 0xf) << 4) | ((b3 >> 2) & 0xf)));
             os.write((byte)(b3 << 6 | b4));
@@ -698,12 +690,12 @@ public class Base64 {
      * @throws IOException
      * @throws Base64DecodingException
      */
-    public static final void decode(InputStream is, OutputStream os) 
+    public static final void decode(InputStream is, OutputStream os)
         throws Base64DecodingException, IOException {
         //byte decodedData[] = null;
-        byte b1 = 0, b2 = 0, b3 = 0, b4 = 0;    
+        byte b1 = 0, b2 = 0, b3 = 0, b4 = 0;
 
-        int index=0;
+        int index = 0;
         byte[] data = new byte[4];
         int read;
         //the begin
@@ -717,13 +709,13 @@ public class Base64 {
                 if (index == 3) {
                     data[index++] = (byte)is.read();
                 }
-                break;   
+                break;
             }
 
             if ((data[index++] = readed) == -1) {
                 //if found "no data" just return null
                 throw new Base64DecodingException("decoding.general");
-            } 
+            }
 
             if (index != 4) {
                 continue;
@@ -737,32 +729,32 @@ public class Base64 {
             os.write((byte)(b1 << 2 | b2 >> 4));
             os.write((byte)(((b2 & 0xf) << 4) | ((b3 >> 2) & 0xf)));
             os.write((byte)(b3 << 6 | b4));
-        }       
+        }
 
         byte d1 = data[0], d2 = data[1], d3 = data[2], d4 = data[3];
         b1 = base64Alphabet[d1];
         b2 = base64Alphabet[d2];
         b3 = base64Alphabet[d3];
         b4 = base64Alphabet[d4];
-        if ((b3 == -1) || (b4 == -1)) { //Check if they are PAD characters
+        if (b3 == -1 || b4 == -1) { //Check if they are PAD characters
             if (isPad(d3) && isPad(d4)) {               //Two PAD e.g. 3c[Pad][Pad]
                 if ((b2 & 0xf) != 0) { //last 4 bits should be zero
                     throw new Base64DecodingException("decoding.general");
                 }
-                os.write((byte)(b1 << 2 | b2 >> 4));                
+                os.write((byte)(b1 << 2 | b2 >> 4));
             } else if (!isPad(d3) && isPad(d4)) {               //One PAD  e.g. 3cQ[Pad]
                 b3 = base64Alphabet[d3];
                 if ((b3 & 0x3) != 0) { //last 2 bits should be zero
                     throw new Base64DecodingException("decoding.general");
                 }
                 os.write((byte)(b1 << 2 | b2 >> 4));
-                os.write((byte)(((b2 & 0xf) << 4) | ((b3 >> 2) & 0xf)));                
+                os.write((byte)(((b2 & 0xf) << 4) | ((b3 >> 2) & 0xf)));
             } else {
                 //an error  like "3c[Pad]r", "3cdX", "3cXd", "3cXX" where X is non data
                 throw new Base64DecodingException("decoding.general");
             }
         } else {
-            //No PAD e.g 3cQl         
+            //No PAD e.g 3cQl
             os.write((byte)(b1 << 2 | b2 >> 4));
             os.write((byte)(((b2 & 0xf) << 4) | ((b3 >> 2) & 0xf)));
             os.write((byte)(b3 << 6 | b4));
@@ -771,7 +763,7 @@ public class Base64 {
 
     /**
      * remove WhiteSpace from MIME containing encoded Base64 data.
-     * 
+     *
      * @param data  the byte array of base64 data (with WS)
      * @return      the new length
      */
