@@ -18,7 +18,9 @@
  */
 package org.apache.xml.security.utils;
 
+import java.io.File;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.OutputStream;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
@@ -32,42 +34,55 @@ import javax.xml.XMLConstants;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
+import javax.xml.validation.Schema;
 
 import org.apache.xml.security.c14n.CanonicalizationException;
 import org.apache.xml.security.c14n.Canonicalizer;
 import org.apache.xml.security.c14n.InvalidCanonicalizerException;
 import org.w3c.dom.Attr;
+import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.NamedNodeMap;
 import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
-import org.w3c.dom.ProcessingInstruction;
 import org.w3c.dom.Text;
+import org.xml.sax.EntityResolver;
+import org.xml.sax.ErrorHandler;
+import org.xml.sax.InputSource;
+import org.xml.sax.SAXException;
 
 /**
  * DOM and XML accessibility and comfort functions.
  *
  * @author Christian Geuer-Pollmann
  */
-public class XMLUtils {
+public final class XMLUtils {
 
     private static boolean ignoreLineBreaks =
         AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
             public Boolean run() {
-                return Boolean.valueOf(Boolean.getBoolean
-                    ("org.apache.xml.security.ignoreLineBreaks"));
+                return Boolean.getBoolean
+                    ("org.apache.xml.security.ignoreLineBreaks");
             }
-        }).booleanValue();
-    
+        });
+
+    @SuppressWarnings("unchecked")
+    private static final WeakObjectPool<DocumentBuilder, ParserConfigurationException> pools[] = new WeakObjectPool[4];
+    static {
+        pools[0] = new DocumentBuilderPool(false, false);
+        pools[1] = new DocumentBuilderPool(false, true);
+        pools[2] = new DocumentBuilderPool(true, false);
+        pools[3] = new DocumentBuilderPool(true, true);
+    }
+
     private static volatile String dsPrefix = "ds";
     private static volatile String ds11Prefix = "dsig11";
     private static volatile String xencPrefix = "xenc";
     private static volatile String xenc11Prefix = "xenc11";
-    
-    /** {@link org.apache.commons.logging} logging facility */
-    private static final org.apache.commons.logging.Log log = 
-        org.apache.commons.logging.LogFactory.getLog(XMLUtils.class);
+
+    private static final org.slf4j.Logger log =
+        org.slf4j.LoggerFactory.getLogger(XMLUtils.class);
 
 
     /**
@@ -77,7 +92,7 @@ public class XMLUtils {
     private XMLUtils() {
         // we don't allow instantiation
     }
-    
+
     /**
      * Set the prefix for the digital signature namespace
      * @param prefix the new prefix for the digital signature namespace
@@ -88,9 +103,9 @@ public class XMLUtils {
         JavaUtils.checkRegisterPermission();
         dsPrefix = prefix;
     }
-    
+
     /**
-     * Set the prefix for the digital signature 1.1 namespace 
+     * Set the prefix for the digital signature 1.1 namespace
      * @param prefix the new prefix for the digital signature 1.1 namespace
      * @throws SecurityException if a security manager is installed and the
      *    caller does not have permission to set the prefix
@@ -99,7 +114,7 @@ public class XMLUtils {
         JavaUtils.checkRegisterPermission();
         ds11Prefix = prefix;
     }
-    
+
     /**
      * Set the prefix for the encryption namespace
      * @param prefix the new prefix for the encryption namespace
@@ -110,7 +125,7 @@ public class XMLUtils {
         JavaUtils.checkRegisterPermission();
         xencPrefix = prefix;
     }
-    
+
     /**
      * Set the prefix for the encryption namespace 1.1
      * @param prefix the new prefix for the encryption namespace 1.1
@@ -121,10 +136,10 @@ public class XMLUtils {
         JavaUtils.checkRegisterPermission();
         xenc11Prefix = prefix;
     }
-    
+
     public static Element getNextElement(Node el) {
         Node node = el;
-        while ((node != null) && (node.getNodeType() != Node.ELEMENT_NODE)) {
+        while (node != null && node.getNodeType() != Node.ELEMENT_NODE) {
             node = node.getNextSibling();
         }
         return (Element)node;
@@ -135,14 +150,14 @@ public class XMLUtils {
      * @param result
      * @param exclude
      * @param com whether comments or not
-     */   
+     */
     public static void getSet(Node rootNode, Set<Node> result, Node exclude, boolean com) {
-        if ((exclude != null) && isDescendantOrSelf(exclude, rootNode)) {
+        if (exclude != null && isDescendantOrSelf(exclude, rootNode)) {
             return;
         }
         getSetRec(rootNode, result, exclude, com);
     }
-    
+
     @SuppressWarnings("fallthrough")
     private static void getSetRec(final Node rootNode, final Set<Node> result,
                                 final Node exclude, final boolean com) {
@@ -155,23 +170,24 @@ public class XMLUtils {
             Element el = (Element)rootNode;
             if (el.hasAttributes()) {
                 NamedNodeMap nl = el.getAttributes();
-                for (int i = 0;i < nl.getLength(); i++) {
+                int length = nl.getLength();
+                for (int i = 0; i < length; i++) {
                     result.add(nl.item(i));
                 }
             }
             //no return keep working
         case Node.DOCUMENT_NODE:   	   			
-            for (Node r = rootNode.getFirstChild(); r != null; r = r.getNextSibling()) {                                    
+            for (Node r = rootNode.getFirstChild(); r != null; r = r.getNextSibling()) {
                 if (r.getNodeType() == Node.TEXT_NODE) {
-                    result.add(r); 
-                    while ((r != null) && (r.getNodeType() == Node.TEXT_NODE)) {
+                    result.add(r);
+                    while (r != null && r.getNodeType() == Node.TEXT_NODE) {
                         r = r.getNextSibling();
                     }
                     if (r == null) {
                         return;
                     }
-                }  
-                getSetRec(r, result, exclude, com);                
+                }
+                getSetRec(r, result, exclude, com);
             }
             return;
         case Node.COMMENT_NODE:
@@ -213,20 +229,20 @@ public class XMLUtils {
             }
 
             os.write(Canonicalizer.getInstance(
-                Canonicalizer.ALGO_ID_C14N_WITH_COMMENTS).canonicalizeSubtree(contextNode)
+                Canonicalizer.ALGO_ID_C14N_PHYSICAL).canonicalizeSubtree(contextNode)
             );
         } catch (IOException ex) {
             if (log.isDebugEnabled()) {
-                log.debug(ex);
+                log.debug(ex.getMessage(), ex);
             }
         }
         catch (InvalidCanonicalizerException ex) {
             if (log.isDebugEnabled()) {
-                log.debug(ex);
+                log.debug(ex.getMessage(), ex);
             }
         } catch (CanonicalizationException ex) {
             if (log.isDebugEnabled()) {
-                log.debug(ex);
+                log.debug(ex.getMessage(), ex);
             }
         }
     }
@@ -251,17 +267,17 @@ public class XMLUtils {
             );
         } catch (IOException ex) {
             if (log.isDebugEnabled()) {
-                log.debug(ex);
+                log.debug(ex.getMessage(), ex);
             }
             // throw new RuntimeException(ex.getMessage());
         } catch (InvalidCanonicalizerException ex) {
             if (log.isDebugEnabled()) {
-                log.debug(ex);
+                log.debug(ex.getMessage(), ex);
             }
             // throw new RuntimeException(ex.getMessage());
         } catch (CanonicalizationException ex) {
             if (log.isDebugEnabled()) {
-                log.debug(ex);
+                log.debug(ex.getMessage(), ex);
             }
             // throw new RuntimeException(ex.getMessage());
         }
@@ -275,7 +291,7 @@ public class XMLUtils {
      */
     public static String getFullTextChildrenFromElement(Element element) {
         StringBuilder sb = new StringBuilder();
-        
+
         Node child = element.getFirstChild();
         while (child != null) {
             if (child.getNodeType() == Node.TEXT_NODE) {
@@ -299,12 +315,12 @@ public class XMLUtils {
             throw new RuntimeException("Document is null");
         }
 
-        if ((dsPrefix == null) || (dsPrefix.length() == 0)) {
+        if (dsPrefix == null || dsPrefix.length() == 0) {
             return doc.createElementNS(Constants.SignatureSpecNS, elementName);
-        } 
+        }
         return doc.createElementNS(Constants.SignatureSpecNS, dsPrefix + ":" + elementName);
     }
-    
+
     /**
      * Creates an Element in the XML Signature 1.1 specification namespace.
      *
@@ -317,9 +333,9 @@ public class XMLUtils {
             throw new RuntimeException("Document is null");
         }
 
-        if ((ds11Prefix == null) || (ds11Prefix.length() == 0)) {
+        if (ds11Prefix == null || ds11Prefix.length() == 0) {
             return doc.createElementNS(Constants.SignatureSpec11NS, elementName);
-        } 
+        }
         return doc.createElementNS(Constants.SignatureSpec11NS, ds11Prefix + ":" + elementName);
     }
 
@@ -335,15 +351,15 @@ public class XMLUtils {
             throw new RuntimeException("Document is null");
         }
 
-        if ((xencPrefix == null) || (xencPrefix.length() == 0)) {
+        if (xencPrefix == null || xencPrefix.length() == 0) {
             return doc.createElementNS(EncryptionConstants.EncryptionSpecNS, elementName);
         }
-        return 
+        return
             doc.createElementNS(
                 EncryptionConstants.EncryptionSpecNS, xencPrefix + ":" + elementName
             );
     }
-    
+
     /**
      * Creates an Element in the XML Encryption 1.1 specification namespace.
      *
@@ -356,10 +372,10 @@ public class XMLUtils {
             throw new RuntimeException("Document is null");
         }
 
-        if ((xenc11Prefix == null) || (xenc11Prefix.length() == 0)) {
+        if (xenc11Prefix == null || xenc11Prefix.length() == 0) {
             return doc.createElementNS(EncryptionConstants.EncryptionSpec11NS, elementName);
         }
-        return 
+        return
             doc.createElementNS(
                 EncryptionConstants.EncryptionSpec11NS, xenc11Prefix + ":" + elementName
             );
@@ -371,7 +387,7 @@ public class XMLUtils {
      *
      * @param element
      * @param localName
-     * @return true if the element is in XML Signature namespace and the local name equals 
+     * @return true if the element is in XML Signature namespace and the local name equals
      * the supplied one
      */
     public static boolean elementIsInSignatureSpace(Element element, String localName) {
@@ -379,17 +395,17 @@ public class XMLUtils {
             return false;
         }
 
-        return Constants.SignatureSpecNS.equals(element.getNamespaceURI()) 
+        return Constants.SignatureSpecNS.equals(element.getNamespaceURI())
             && element.getLocalName().equals(localName);
     }
-    
+
     /**
      * Returns true if the element is in XML Signature 1.1 namespace and the local
      * name equals the supplied one.
      *
      * @param element
      * @param localName
-     * @return true if the element is in XML Signature namespace and the local name equals 
+     * @return true if the element is in XML Signature namespace and the local name equals
      * the supplied one
      */
     public static boolean elementIsInSignature11Space(Element element, String localName) {
@@ -397,7 +413,7 @@ public class XMLUtils {
             return false;
         }
 
-        return Constants.SignatureSpec11NS.equals(element.getNamespaceURI()) 
+        return Constants.SignatureSpec11NS.equals(element.getNamespaceURI())
             && element.getLocalName().equals(localName);
     }
 
@@ -407,31 +423,31 @@ public class XMLUtils {
      *
      * @param element
      * @param localName
-     * @return true if the element is in XML Encryption namespace and the local name 
+     * @return true if the element is in XML Encryption namespace and the local name
      * equals the supplied one
      */
     public static boolean elementIsInEncryptionSpace(Element element, String localName) {
         if (element == null){
             return false;
         }
-        return EncryptionConstants.EncryptionSpecNS.equals(element.getNamespaceURI()) 
+        return EncryptionConstants.EncryptionSpecNS.equals(element.getNamespaceURI())
             && element.getLocalName().equals(localName);
     }
-    
+
     /**
      * Returns true if the element is in XML Encryption 1.1 namespace and the local
      * name equals the supplied one.
      *
      * @param element
      * @param localName
-     * @return true if the element is in XML Encryption 1.1 namespace and the local name 
+     * @return true if the element is in XML Encryption 1.1 namespace and the local name
      * equals the supplied one
      */
     public static boolean elementIsInEncryption11Space(Element element, String localName) {
         if (element == null){
             return false;
         }
-        return EncryptionConstants.EncryptionSpec11NS.equals(element.getNamespaceURI()) 
+        return EncryptionConstants.EncryptionSpec11NS.equals(element.getNamespaceURI())
             && element.getLocalName().equals(localName);
     }
 
@@ -447,7 +463,7 @@ public class XMLUtils {
     public static Document getOwnerDocument(Node node) {
         if (node.getNodeType() == Node.DOCUMENT_NODE) {
             return (Document) node;
-        } 
+        }
         try {
             return node.getOwnerDocument();
         } catch (NullPointerException npe) {
@@ -464,7 +480,7 @@ public class XMLUtils {
      * if the {@link Node} is a {@link Document}.
      *
      * @param xpathNodeSet
-     * @return the owner document 
+     * @return the owner document
      */
     public static Document getOwnerDocument(Set<Node> xpathNodeSet) {
         NullPointerException npe = null;
@@ -472,17 +488,17 @@ public class XMLUtils {
             int nodeType = node.getNodeType();
             if (nodeType == Node.DOCUMENT_NODE) {
                 return (Document) node;
-            } 
+            }
             try {
                 if (nodeType == Node.ATTRIBUTE_NODE) {
-                    return ((Attr)node).getOwnerElement().getOwnerDocument();  
+                    return ((Attr)node).getOwnerElement().getOwnerDocument();
                 }
                 return node.getOwnerDocument();
             } catch (NullPointerException e) {
                 npe = e;
             }
         }
-        
+
         throw new NullPointerException(I18n.translate("endorsed.jdk1.4.0")
                                        + " Original message was \""
                                        + (npe == null ? "" : npe.getMessage()) + "\"");
@@ -497,7 +513,7 @@ public class XMLUtils {
      * @return the element.
      */
     public static Element createDSctx(Document doc, String prefix, String namespace) {
-        if ((prefix == null) || (prefix.trim().length() == 0)) {
+        if (prefix == null || prefix.trim().length() == 0) {
             throw new IllegalArgumentException("You must supply a prefix");
         }
 
@@ -600,12 +616,12 @@ public class XMLUtils {
                 if (!element.hasChildNodes()) {
                     break;
                 }
-                if (element.hasAttributes()) {            	 
+                if (element.hasAttributes()) {            	
                     NamedNodeMap attributes = element.getAttributes();         	
-                    int attributesLength = attributes.getLength();    
+                    int attributesLength = attributes.getLength();
 
-                    for (Node child = element.getFirstChild(); child!=null; 
-                        child = child.getNextSibling()) {            
+                    for (Node child = element.getFirstChild(); child!=null;
+                        child = child.getNextSibling()) {
 
                         if (child.getNodeType() != Node.ELEMENT_NODE) {
                             continue;
@@ -613,11 +629,11 @@ public class XMLUtils {
                         Element childElement = (Element) child;
 
                         for (int i = 0; i < attributesLength; i++) {
-                            Attr currentAttr = (Attr) attributes.item(i); 
+                            Attr currentAttr = (Attr) attributes.item(i);
                             if (!namespaceNs.equals(currentAttr.getNamespaceURI())) {
                                 continue;
                             }
-                            if (childElement.hasAttributeNS(namespaceNs, 
+                            if (childElement.hasAttributeNS(namespaceNs,
                                                             currentAttr.getLocalName())) {
                                 continue;
                             }
@@ -625,7 +641,7 @@ public class XMLUtils {
                                                         currentAttr.getName(),
                                                         currentAttr.getNodeValue());         					
                         }
-                    }            
+                    }
                 }
             case Node.ENTITY_REFERENCE_NODE :
             case Node.DOCUMENT_NODE :
@@ -633,7 +649,7 @@ public class XMLUtils {
                 sibling = node.getFirstChild();
                 break;
             }
-            while ((sibling == null) && (parent != null)) {
+            while (sibling == null && parent != null) {
                 sibling = parent.getNextSibling();
                 parent = parent.getParentNode();
             }
@@ -645,7 +661,7 @@ public class XMLUtils {
             sibling = node.getNextSibling();
         } while (true);
     }
-    
+
     /**
      * @param sibling
      * @param nodeName
@@ -654,7 +670,7 @@ public class XMLUtils {
      */
     public static Element selectDsNode(Node sibling, String nodeName, int number) {
         while (sibling != null) {
-            if (Constants.SignatureSpecNS.equals(sibling.getNamespaceURI()) 
+            if (Constants.SignatureSpecNS.equals(sibling.getNamespaceURI())
                 && sibling.getLocalName().equals(nodeName)) {
                 if (number == 0){
                     return (Element)sibling;
@@ -674,7 +690,7 @@ public class XMLUtils {
      */
     public static Element selectDs11Node(Node sibling, String nodeName, int number) {
         while (sibling != null) {
-            if (Constants.SignatureSpec11NS.equals(sibling.getNamespaceURI()) 
+            if (Constants.SignatureSpec11NS.equals(sibling.getNamespaceURI())
                 && sibling.getLocalName().equals(nodeName)) {
                 if (number == 0){
                     return (Element)sibling;
@@ -694,7 +710,7 @@ public class XMLUtils {
      */
     public static Element selectXencNode(Node sibling, String nodeName, int number) {
         while (sibling != null) {
-            if (EncryptionConstants.EncryptionSpecNS.equals(sibling.getNamespaceURI()) 
+            if (EncryptionConstants.EncryptionSpecNS.equals(sibling.getNamespaceURI())
                 && sibling.getLocalName().equals(nodeName)) {
                 if (number == 0){
                     return (Element)sibling;
@@ -724,7 +740,7 @@ public class XMLUtils {
         }
         return (Text)n;
     }
-    
+
     /**
      * @param sibling
      * @param nodeName
@@ -771,7 +787,7 @@ public class XMLUtils {
      */
     public static Element selectNode(Node sibling, String uri, String nodeName, int number) {
         while (sibling != null) {
-            if (sibling.getNamespaceURI() != null && sibling.getNamespaceURI().equals(uri) 
+            if (sibling.getNamespaceURI() != null && sibling.getNamespaceURI().equals(uri)
                 && sibling.getLocalName().equals(nodeName)) {
                 if (number == 0){
                     return (Element)sibling;
@@ -785,22 +801,22 @@ public class XMLUtils {
 
     /**
      * @param sibling
-     * @param nodeName    
+     * @param nodeName
      * @return nodes with the constrain
      */
     public static Element[] selectDsNodes(Node sibling, String nodeName) {
         return selectNodes(sibling, Constants.SignatureSpecNS, nodeName);
     }
-    
+
     /**
      * @param sibling
-     * @param nodeName    
+     * @param nodeName
      * @return nodes with the constrain
      */
     public static Element[] selectDs11Nodes(Node sibling, String nodeName) {
         return selectNodes(sibling, Constants.SignatureSpec11NS, nodeName);
     }
-    
+
     /**
      * @param sibling
      * @param uri
@@ -810,7 +826,7 @@ public class XMLUtils {
     public static Element[] selectNodes(Node sibling, String uri, String nodeName) {
         List<Element> list = new ArrayList<Element>();
         while (sibling != null) {
-            if (sibling.getNamespaceURI() != null && sibling.getNamespaceURI().equals(uri) 
+            if (sibling.getNamespaceURI() != null && sibling.getNamespaceURI().equals(uri)
                 && sibling.getLocalName().equals(nodeName)) {
                 list.add((Element)sibling);
             }
@@ -837,7 +853,7 @@ public class XMLUtils {
         }
         return resultSet;
     }
-    
+
     /**
      * Method getStrFromNode
      *
@@ -860,9 +876,9 @@ public class XMLUtils {
 
             return sb.toString();
         } else if (xpathnode.getNodeType() == Node.ATTRIBUTE_NODE) {
-            return ((Attr) xpathnode).getNodeValue();
+            return xpathnode.getNodeValue();
         } else if (xpathnode.getNodeType() == Node.PROCESSING_INSTRUCTION_NODE) {
-            return ((ProcessingInstruction) xpathnode).getNodeValue();
+            return xpathnode.getNodeValue();
         }
 
         return null;
@@ -903,10 +919,10 @@ public class XMLUtils {
     public static boolean ignoreLineBreaks() {
         return ignoreLineBreaks;
     }
-    
+
     /**
      * Returns the attribute value for the attribute with the specified name.
-     * Returns null if there is no such attribute, or 
+     * Returns null if there is no such attribute, or
      * the empty string if the attribute value is empty.
      *
      * <p>This works around a limitation of the DOM
@@ -922,7 +938,7 @@ public class XMLUtils {
         Attr attr = elem.getAttributeNodeNS(null, name);
         return (attr == null) ? null : attr.getValue();
     }
-    
+
     /**
      * This method is a tree-search to help prevent against wrapping attacks. It checks that no
      * two Elements have ID Attributes that match the "value" argument, if this is the case then
@@ -930,22 +946,26 @@ public class XMLUtils {
      * a matching Element has been found, just that no wrapping attack has been detected.
      */
     public static boolean protectAgainstWrappingAttack(Node startNode, String value) {
-        Node startParent = startNode.getParentNode();
+        String id = value.trim();
+        if (!id.isEmpty() && id.charAt(0) == '#') {
+            id = id.substring(1);
+        }
+
+        Node startParent = null;
         Node processedNode = null;
         Element foundElement = null;
-        
-        String id = value.trim();
-        if (!"".equals(id) && id.charAt(0) == '#') {
-            id = id.substring(1);
+        if (startNode != null) {
+            startParent = startNode.getParentNode();
         }
 
         while (startNode != null) {
             if (startNode.getNodeType() == Node.ELEMENT_NODE) {
                 Element se = (Element) startNode;
-                
+
                 NamedNodeMap attributes = se.getAttributes();
                 if (attributes != null) {
-                    for (int i = 0; i < attributes.getLength(); i++) {
+                    int length = attributes.getLength();
+                    for (int i = 0; i < length; i++) {
                         Attr attr = (Attr)attributes.item(i);
                         if (attr.isId() && id.equals(attr.getValue())) {
                             if (foundElement == null) {
@@ -968,7 +988,7 @@ public class XMLUtils {
                 // close node processing, get sibling
                 startNode = processedNode.getNextSibling();
             }
-            
+
             // no more siblings, get parent, all children
             // of parent are processed.
             while (startNode == null) {
@@ -982,30 +1002,34 @@ public class XMLUtils {
         }
         return true;
     }
-    
+
     /**
      * This method is a tree-search to help prevent against wrapping attacks. It checks that no other
-     * Element than the given "knownElement" argument has an ID attribute that matches the "value" 
+     * Element than the given "knownElement" argument has an ID attribute that matches the "value"
      * argument, which is the ID value of "knownElement". If this is the case then "false" is returned.
      */
     public static boolean protectAgainstWrappingAttack(
         Node startNode, Element knownElement, String value
     ) {
-        Node startParent = startNode.getParentNode();
-        Node processedNode = null;
-        
         String id = value.trim();
-        if (!"".equals(id) && id.charAt(0) == '#') {
+        if (!id.isEmpty() && id.charAt(0) == '#') {
             id = id.substring(1);
         }
 
+        Node startParent = null;
+        Node processedNode = null;
+        if (startNode != null) {
+            startParent = startNode.getParentNode();
+        }
+
         while (startNode != null) {
             if (startNode.getNodeType() == Node.ELEMENT_NODE) {
                 Element se = (Element) startNode;
-                
+
                 NamedNodeMap attributes = se.getAttributes();
                 if (attributes != null) {
-                    for (int i = 0; i < attributes.getLength(); i++) {
+                    int length = attributes.getLength();
+                    for (int i = 0; i < length; i++) {
                         Attr attr = (Attr)attributes.item(i);
                         if (attr.isId() && id.equals(attr.getValue()) && se != knownElement) {
                             log.debug("Multiple elements with the same 'Id' attribute value!");
@@ -1023,7 +1047,7 @@ public class XMLUtils {
                 // close node processing, get sibling
                 startNode = processedNode.getNextSibling();
             }
-            
+
             // no more siblings, get parent, all children
             // of parent are processed.
             while (startNode == null) {
@@ -1037,23 +1061,151 @@ public class XMLUtils {
         }
         return true;
     }
-    
+
     public static DocumentBuilder createDocumentBuilder(boolean validating) throws ParserConfigurationException {
         return createDocumentBuilder(validating, true);
     }
-    
+
     public static DocumentBuilder createDocumentBuilder(
         boolean validating, boolean disAllowDocTypeDeclarations
     ) throws ParserConfigurationException {
-        DocumentBuilderFactory dfactory = DocumentBuilderFactory.newInstance();
-        dfactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, Boolean.TRUE);
-        if (disAllowDocTypeDeclarations) {
-            dfactory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
+        int idx = getPoolsIndex(validating, disAllowDocTypeDeclarations);
+        return pools[idx].getObject();
+    }
+
+
+    /**
+     * Return this document builder to be reused
+     * @param db DocumentBuilder returned from any of {@link #createDocumentBuilder} methods.
+     * @return whether it was successfully returned to the pool
+     */
+    public static boolean repoolDocumentBuilder(DocumentBuilder db) {
+        if (!(db instanceof DocumentBuilderProxy)) {
+            return false;
         }
-        dfactory.setValidating(validating);        
-        dfactory.setNamespaceAware(true);
-        
-        return dfactory.newDocumentBuilder();
+        db.reset();
+        boolean disAllowDocTypeDeclarations =
+            ((DocumentBuilderProxy)db).disAllowDocTypeDeclarations();
+        int idx = getPoolsIndex(db.isValidating(), disAllowDocTypeDeclarations);
+        return pools[idx].repool(db);
+    }
+
+
+    /**
+     * We need this proxy wrapping DocumentBuilder to record the value
+     * passed to disAllowDoctypeDeclarations.  It's needed to figure out
+     * on which pool to return.
+     */
+    private static class DocumentBuilderProxy extends DocumentBuilder {
+        private final DocumentBuilder delegate;
+        private final boolean disAllowDocTypeDeclarations;
+
+        private DocumentBuilderProxy(DocumentBuilder actual, boolean disAllowDocTypeDeclarations) {
+            delegate = actual;
+            this.disAllowDocTypeDeclarations = disAllowDocTypeDeclarations;
+        }
+
+        boolean disAllowDocTypeDeclarations() {
+            return disAllowDocTypeDeclarations;
+        }
+
+        public void reset() {
+            delegate.reset();
+        }
+
+        public Document parse(InputStream is) throws SAXException, IOException {
+            return delegate.parse(is);
+        }
+
+        public Document parse(InputStream is, String systemId)
+                throws SAXException, IOException {
+            return delegate.parse(is, systemId);
+        }
+
+        public Document parse(String uri) throws SAXException, IOException {
+            return delegate.parse(uri);
+        }
+
+        public Document parse(File f) throws SAXException, IOException {
+            return delegate.parse(f);
+        }
+
+        public Schema getSchema() {
+            return delegate.getSchema();
+        }
+
+        public boolean isXIncludeAware() {
+            return delegate.isXIncludeAware();
+        }
+
+        @Override
+        public Document parse(InputSource is) throws SAXException, IOException {
+            return delegate.parse(is);
+        }
+
+        @Override
+        public boolean isNamespaceAware() {
+            return delegate.isNamespaceAware();
+        }
+
+        @Override
+        public boolean isValidating() {
+            return delegate.isValidating();
+        }
+
+        @Override
+        public void setEntityResolver(EntityResolver er) {
+            delegate.setEntityResolver(er);
+        }
+
+        @Override
+        public void setErrorHandler(ErrorHandler eh) {
+            delegate.setErrorHandler(eh);
+        }
+
+        @Override
+        public Document newDocument() {
+            return delegate.newDocument();
+        }
+
+        @Override
+        public DOMImplementation getDOMImplementation() {
+            return delegate.getDOMImplementation();
+        }
+
+    }
+
+    private static final class DocumentBuilderPool
+        extends WeakObjectPool<DocumentBuilder, ParserConfigurationException> {
+
+        private final boolean validating, disAllowDocTypeDeclarations;
+
+        public DocumentBuilderPool(boolean validating, boolean disAllowDocTypeDeclarations) {
+            this.validating = validating;
+            this.disAllowDocTypeDeclarations = disAllowDocTypeDeclarations;
+        }
+
+        @Override
+        protected DocumentBuilder createObject() throws ParserConfigurationException {
+            DocumentBuilderFactory dfactory = DocumentBuilderFactory.newInstance();
+            dfactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, Boolean.TRUE);
+            if (disAllowDocTypeDeclarations) {
+                dfactory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
+            }
+            dfactory.setValidating(validating);
+            dfactory.setNamespaceAware(true);
+            return new DocumentBuilderProxy(dfactory.newDocumentBuilder(), disAllowDocTypeDeclarations);
+        }
+    }
+
+    /**
+     * Maps the two boolean configuration options for the factories to the array index for the WeakObjectPool
+     * @param validating
+     * @param disAllowDocTypeDeclarations
+     * @return the index to the {@link #pools}
+     */
+    private static int getPoolsIndex(boolean validating, boolean disAllowDocTypeDeclarations) {
+        return (validating ? 2 : 0) + (disAllowDocTypeDeclarations ? 1 : 0);
     }
 
 }
