@@ -55,7 +55,7 @@ public class XMLSignatureTest extends org.junit.Assert {
     private Key[] VALIDATE_KEYS;
     private SignatureMethod[] SIG_METHODS;
     private URIDereferencer ud;
-    private final static String DSA_SHA256 =
+    private static final String DSA_SHA256 =
         "http://www.w3.org/2009/xmldsig11#dsa-sha256";
 
     static {
@@ -69,7 +69,7 @@ public class XMLSignatureTest extends org.junit.Assert {
         kifac = KeyInfoFactory.getInstance
             ("DOM", new org.apache.jcp.xml.dsig.internal.dom.XMLDSigRI());
 
-        // set up the corresponding SignatureMethod 
+        // set up the corresponding SignatureMethod
         SIG_METHODS = new SignatureMethod[3];
         SIG_METHODS[0] = fac.newSignatureMethod(SignatureMethod.DSA_SHA1, null);
         SIG_METHODS[1] = fac.newSignatureMethod(SignatureMethod.RSA_SHA1, null);
@@ -92,6 +92,7 @@ public class XMLSignatureTest extends org.junit.Assert {
         ud = new LocalHttpCacheURIDereferencer();
     }
 
+    @SuppressWarnings("rawtypes")
     @org.junit.Test
     public void testConstructor() throws Exception {
         XMLSignature sig = null;
@@ -112,17 +113,19 @@ public class XMLSignatureTest extends org.junit.Assert {
                 fail("Should throw a NPE for null references");
             } catch (NullPointerException npe) {
             } catch (Exception ex) {
-                fail("Should throw a NPE instead of " + ex + 
+                fail("Should throw a NPE instead of " + ex +
                      " for null references");
             }
         }
         try {
-            sig = fac.newXMLSignature(defSi, defKi, 
-                     Collections.singletonList("wrongType"), id, sigValueId);
+            // use raw List type to test for invalid entries
+            List invalidObjects = new ArrayList();
+            addEntryToRawList(invalidObjects, "wrongType");
+            fac.newXMLSignature(defSi, defKi, invalidObjects, id, sigValueId);
             fail("Should throw a CCE for invalid objects");
         } catch (ClassCastException cce) {
         } catch (Exception ex) {
-            fail("Should throw a CCE instead of " + ex + 
+            fail("Should throw a CCE instead of " + ex +
                  " for invalid objects");
         }
         sig = fac.newXMLSignature(defSi, defKi, objs, id, sigValueId);
@@ -141,17 +144,17 @@ public class XMLSignatureTest extends org.junit.Assert {
         assertNull(sig.getSignatureValue().getId());
         assertEquals(sig.getSignedInfo(), defSi);
     }
-    
+
     @org.junit.Test
     public void testisFeatureSupported() throws Exception {
 
         XMLSignature sig = fac.newXMLSignature(defSi, null);
-        
+
         try {
-            sig.isFeatureSupported(null); 
-            fail("Should raise a NPE for null feature"); 
+            sig.isFeatureSupported(null);
+            fail("Should raise a NPE for null feature");
         } catch (NullPointerException npe) {}
-            
+
         assertTrue(!sig.isFeatureSupported("not supported"));
     }
 
@@ -172,11 +175,12 @@ public class XMLSignatureTest extends org.junit.Assert {
                 ki = kifac.newKeyInfo(Collections.singletonList
                     (kifac.newKeyName("testuser")));
             }
-            sig = fac.newXMLSignature(si, ki, objs, id, sigValueId); 
+            sig = fac.newXMLSignature(si, ki, objs, id, sigValueId);
             Document doc = TestUtils.newDocument();
             signContext = new DOMSignContext(SIGN_KEYS[i], doc);
             signContext.setURIDereferencer(ud);
             sig.sign(signContext);
+            TestUtils.validateSecurityOrEncryptionElement(doc.getDocumentElement());
             validateContext = new DOMValidateContext
                 (VALIDATE_KEYS[i], doc.getDocumentElement());
             validateContext.setURIDereferencer(ud);
@@ -204,7 +208,7 @@ public class XMLSignatureTest extends org.junit.Assert {
                 ki = kifac.newKeyInfo(Collections.singletonList
                     (kifac.newKeyName("testuser")));
             }
-            sig = fac.newXMLSignature(si, ki, objs, id, sigValueId); 
+            sig = fac.newXMLSignature(si, ki, objs, id, sigValueId);
             Document doc = TestUtils.newDocument();
             signContext = new DOMSignContext(SIGN_KEYS[i], doc);
             signContext.setProperty
@@ -212,10 +216,11 @@ public class XMLSignatureTest extends org.junit.Assert {
             signContext.setURIDereferencer(ud);
             try {
                 sig.sign(signContext);
+                // note - don't bother validating the returned XML here, because there shouldn't be any.
                 fail("Should have failed because TestProvider does not " +
                      "support " + SIGN_KEYS[i].getAlgorithm());
             } catch (Exception e) {
-                assertTrue(e.getMessage(), 
+                assertTrue(e.getMessage(),
                     e.getCause() instanceof NoSuchAlgorithmException);
             }
         }
@@ -226,12 +231,13 @@ public class XMLSignatureTest extends org.junit.Assert {
         SignedInfo si = createSignedInfo(SIG_METHODS[1]);
         KeyInfo	ki = kifac.newKeyInfo(Collections.singletonList
                     (kifac.newKeyValue((PublicKey) VALIDATE_KEYS[1])));
-        XMLSignature sig = fac.newXMLSignature(si, ki, objs, id, sigValueId); 
+        XMLSignature sig = fac.newXMLSignature(si, ki, objs, id, sigValueId);
         Document doc = TestUtils.newDocument();
         XMLSignContext signContext = new DOMSignContext(SIGN_KEYS[1], doc);
         signContext.putNamespacePrefix(XMLSignature.XMLNS, "");
         signContext.setURIDereferencer(ud);
         sig.sign(signContext);
+        TestUtils.validateSecurityOrEncryptionElement(doc.getDocumentElement());
 /*
         StringWriter sw = new StringWriter();
         dumpDocument(doc, sw);
@@ -275,6 +281,7 @@ public class XMLSignatureTest extends org.junit.Assert {
         DOMSignContext dsc = new DOMSignContext(SIGN_KEYS[1], doc);
 
         sig.sign(dsc);
+        TestUtils.validateSecurityOrEncryptionElement(doc.getDocumentElement());
 
 /*
         StringWriter sw = new StringWriter();
@@ -299,7 +306,7 @@ public class XMLSignatureTest extends org.junit.Assert {
     public void testSignTemplateWithObjectNSDefs() throws Exception {
         String base = System.getProperty("basedir") == null ? "./"
                       : System.getProperty("basedir");
- 
+
         File f = new File(base + "/src/test/resources/javax/xml/crypto/dsig/" +
             "signature-enveloping-rsa-template.xml");
 
@@ -326,12 +333,14 @@ public class XMLSignatureTest extends org.junit.Assert {
         // remove the signature node (since it will get recreated)
         parent.removeChild(domSignature.getNode());
         newSignature.sign(signContext);
+        TestUtils.validateSecurityOrEncryptionElement(parent.getLastChild());
 
         // check that Object element retained namespace definitions
         Element objElem = (Element)parent.getFirstChild().getLastChild();
         Attr a = objElem.getAttributeNode("xmlns:test");
-        if (!a.getValue().equals("http://www.example.org/ns"))
+        if (!a.getValue().equals("http://www.example.org/ns")) {
             throw new Exception("Object namespace definition not retained");
+        }
     }
 
     @org.junit.Test
@@ -378,7 +387,7 @@ public class XMLSignatureTest extends org.junit.Assert {
         SignedInfo si = createSignedInfo(sm);
         KeyInfo ki = kifac.newKeyInfo(Collections.singletonList
             (kifac.newKeyValue((PublicKey)TestUtils.getPublicKey("DSA", 2048))));
-        XMLSignature sig = fac.newXMLSignature(si, ki, objs, id, sigValueId); 
+        XMLSignature sig = fac.newXMLSignature(si, ki, objs, id, sigValueId);
         Document doc = TestUtils.newDocument();
         XMLSignContext signContext =
             new DOMSignContext(TestUtils.getPrivateKey("DSA", 2048), doc);
@@ -393,7 +402,7 @@ public class XMLSignatureTest extends org.junit.Assert {
     private SignedInfo createSignedInfo(SignatureMethod sm) throws Exception {
         // set up the building blocks
         CanonicalizationMethod cm = fac.newCanonicalizationMethod
-            (CanonicalizationMethod.INCLUSIVE_WITH_COMMENTS, 
+            (CanonicalizationMethod.INCLUSIVE_WITH_COMMENTS,
              (C14NMethodParameterSpec) null);
         DigestMethod dm = fac.newDigestMethod(DigestMethod.SHA1, null);
         List<Reference> refs = Collections.singletonList(fac.newReference
@@ -401,6 +410,13 @@ public class XMLSignatureTest extends org.junit.Assert {
         return fac.newSignedInfo(cm, sm, refs);
     }
 
+    @SuppressWarnings({
+     "unchecked", "rawtypes"
+    })
+    private static void addEntryToRawList(List list, Object entry) {
+        list.add(entry);
+    }
+
     static class TestProvider extends Provider {
         private static final long serialVersionUID = 1L;
 
