@@ -18,9 +18,6 @@
  */
 package org.apache.xml.security.test.dom.algorithms;
 
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
 import java.io.InputStream;
 import java.lang.reflect.Constructor;
 import java.security.Key;
@@ -38,10 +35,11 @@ import javax.xml.xpath.XPathConstants;
 import javax.xml.xpath.XPathFactory;
 
 import org.apache.xml.security.signature.XMLSignature;
-import org.apache.xml.security.test.DSNamespaceContext;
+import org.apache.xml.security.test.dom.DSNamespaceContext;
 import org.apache.xml.security.transforms.Transforms;
 import org.apache.xml.security.utils.XMLUtils;
 import org.junit.Assert;
+import org.junit.BeforeClass;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.NodeList;
@@ -49,19 +47,19 @@ import org.w3c.dom.NodeList;
 /**
  * A test to make sure that the various Public Key Signature algorithms are working
  */
-public class PKSignatureAlgorithmTest extends org.junit.Assert {
-    
-    private KeyPair rsaKeyPair, ecKeyPair;
+public class PKSignatureAlgorithmTest extends Assert {
+
+    private static KeyPair rsaKeyPair, ecKeyPair;
+    private static boolean bcInstalled;
 
     static {
         org.apache.xml.security.Init.init();
     }
-    
-    private boolean bouncyCastleAvailable = true;
-    
-    public PKSignatureAlgorithmTest() throws Exception {
+
+    @BeforeClass
+    public static void setup() throws Exception {
         //
-        // If the BouncyCastle provider is not installed, then try to load it 
+        // If the BouncyCastle provider is not installed, then try to load it
         // via reflection.
         //
         if (Security.getProvider("BC") == null) {
@@ -72,279 +70,428 @@ public class PKSignatureAlgorithmTest extends org.junit.Assert {
             } catch (Exception e) {
                 //ignore
             }
-            if (cons == null) {
-                bouncyCastleAvailable = false;
-                // BouncyCastle is not available so just return
-                return;
-            } else {
-                Provider provider = (java.security.Provider)cons.newInstance();
+            if (cons != null) {
+                Provider provider = (Provider)cons.newInstance();
                 Security.insertProviderAt(provider, 2);
+                bcInstalled = true;
             }
         }
-        
-        rsaKeyPair = KeyPairGenerator.getInstance("RSA").genKeyPair();
+
+        KeyPairGenerator rsaKpg = KeyPairGenerator.getInstance("RSA");
+        rsaKpg.initialize(2048);
+        rsaKeyPair = rsaKpg.genKeyPair();
+
         ecKeyPair = KeyPairGenerator.getInstance("EC").genKeyPair();
     }
-    
+
+    @org.junit.AfterClass
+    public static void cleanup() throws Exception {
+        Security.removeProvider("BC");
+    }
+
     @org.junit.Test
     public void testRSA_MD5() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
-            return;
-        }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_NOT_RECOMMENDED_RSA_MD5, document, localNames, rsaKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, rsaKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
     public void testRSA_SHA1() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
-            return;
-        }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1, document, localNames, rsaKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, rsaKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
     public void testRSA_SHA_224() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
-            return;
-        }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA224, document, localNames, rsaKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, rsaKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
     public void testRSA_SHA_256() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
-            return;
-        }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA256, document, localNames, rsaKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, rsaKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
     public void testRSA_SHA_384() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
-            return;
-        }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA384, document, localNames, rsaKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, rsaKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
     public void testRSA_SHA_512() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
-            return;
-        }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA512, document, localNames, rsaKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, rsaKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
     public void testRSA_RIPEMD160() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
+        if (!bcInstalled) {
             return;
         }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_RIPEMD160, document, localNames, rsaKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, rsaKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
-    public void testECDSA_SHA1() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
+    public void testRSA_SHA1_MGF1() throws Exception {
+        if (!bcInstalled) {
             return;
         }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
+        sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1_MGF1, document, localNames, rsaKeyPair.getPrivate());
+        // XMLUtils.outputDOM(document, System.out);
+        verify(document, rsaKeyPair.getPublic(), localNames);
+    }
+
+    @org.junit.Test
+    public void testRSA_SHA224_MGF1() throws Exception {
+        if (!bcInstalled) {
+            return;
+        }
+        // Read in plaintext document
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
+        DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
+        Document document = builder.parse(sourceDocument);
+
+        List<String> localNames = new ArrayList<String>();
+        localNames.add("PaymentInfo");
+
+        sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA224_MGF1, document, localNames, rsaKeyPair.getPrivate());
+        // XMLUtils.outputDOM(document, System.out);
+        verify(document, rsaKeyPair.getPublic(), localNames);
+    }
+
+    @org.junit.Test
+    public void testRSA_SHA256_MGF1() throws Exception {
+        if (!bcInstalled) {
+            return;
+        }
+        // Read in plaintext document
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
+        DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
+        Document document = builder.parse(sourceDocument);
+
+        List<String> localNames = new ArrayList<String>();
+        localNames.add("PaymentInfo");
+
+        sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA256_MGF1, document, localNames, rsaKeyPair.getPrivate());
+        // XMLUtils.outputDOM(document, System.out);
+        verify(document, rsaKeyPair.getPublic(), localNames);
+    }
+
+    @org.junit.Test
+    public void testRSA_SHA384_MGF1() throws Exception {
+        if (!bcInstalled) {
+            return;
+        }
+        // Read in plaintext document
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
+        DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
+        Document document = builder.parse(sourceDocument);
+
+        List<String> localNames = new ArrayList<String>();
+        localNames.add("PaymentInfo");
+
+        sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA384_MGF1, document, localNames, rsaKeyPair.getPrivate());
+        // XMLUtils.outputDOM(document, System.out);
+        verify(document, rsaKeyPair.getPublic(), localNames);
+    }
+
+    @org.junit.Test
+    public void testRSA_SHA512_MGF1() throws Exception {
+        if (!bcInstalled) {
+            return;
+        }
+        // Read in plaintext document
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
+        DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
+        Document document = builder.parse(sourceDocument);
+
+        List<String> localNames = new ArrayList<String>();
+        localNames.add("PaymentInfo");
+
+        sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA512_MGF1, document, localNames, rsaKeyPair.getPrivate());
+        // XMLUtils.outputDOM(document, System.out);
+        verify(document, rsaKeyPair.getPublic(), localNames);
+    }
+
+    @org.junit.Test
+    public void testRSA_SHA3_224_MGF1() throws Exception {
+        org.junit.Assume.assumeTrue(bcInstalled);
+
+        // Read in plaintext document
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
+        DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
+        Document document = builder.parse(sourceDocument);
+
+        List<String> localNames = new ArrayList<String>();
+        localNames.add("PaymentInfo");
+
+        sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_224_MGF1, document, localNames, rsaKeyPair.getPrivate());
+        // XMLUtils.outputDOM(document, System.out);
+        verify(document, rsaKeyPair.getPublic(), localNames);
+    }
+
+    @org.junit.Test
+    public void testRSA_SHA3_256_MGF1() throws Exception {
+        org.junit.Assume.assumeTrue(bcInstalled);
+
+        // Read in plaintext document
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
+        DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
+        Document document = builder.parse(sourceDocument);
+
+        List<String> localNames = new ArrayList<String>();
+        localNames.add("PaymentInfo");
+
+        sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_256_MGF1, document, localNames, rsaKeyPair.getPrivate());
+        // XMLUtils.outputDOM(document, System.out);
+        verify(document, rsaKeyPair.getPublic(), localNames);
+    }
+
+    @org.junit.Test
+    public void testRSA_SHA3_384_MGF1() throws Exception {
+        org.junit.Assume.assumeTrue(bcInstalled);
+
+        // Read in plaintext document
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
+        DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
+        Document document = builder.parse(sourceDocument);
+
+        List<String> localNames = new ArrayList<String>();
+        localNames.add("PaymentInfo");
+
+        sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_384_MGF1, document, localNames, rsaKeyPair.getPrivate());
+        // XMLUtils.outputDOM(document, System.out);
+        verify(document, rsaKeyPair.getPublic(), localNames);
+    }
+
+    @org.junit.Test
+    public void testRSA_SHA3_512_MGF1() throws Exception {
+        org.junit.Assume.assumeTrue(bcInstalled);
+
+        // Read in plaintext document
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
+        DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
+        Document document = builder.parse(sourceDocument);
+
+        List<String> localNames = new ArrayList<String>();
+        localNames.add("PaymentInfo");
+
+        sign(XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA3_512_MGF1, document, localNames, rsaKeyPair.getPrivate());
+        // XMLUtils.outputDOM(document, System.out);
+        verify(document, rsaKeyPair.getPublic(), localNames);
+    }
+
+    @org.junit.Test
+    public void testECDSA_SHA1() throws Exception {
+        // Read in plaintext document
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
+        DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
+        Document document = builder.parse(sourceDocument);
+
+        List<String> localNames = new ArrayList<String>();
+        localNames.add("PaymentInfo");
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA1, document, localNames, ecKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, ecKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
     public void testECDSA_SHA_224() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
-            return;
-        }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA224, document, localNames, ecKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, ecKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
     public void testECDSA_SHA_256() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
-            return;
-        }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA256, document, localNames, ecKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, ecKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
     public void testECDSA_SHA_384() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
-            return;
-        }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA384, document, localNames, ecKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, ecKeyPair.getPublic(), localNames);
     }
-    
+
     @org.junit.Test
     public void testECDSA_SHA_512() throws Exception {
-        
-        if (!bouncyCastleAvailable) {
-            return;
-        }
-        
         // Read in plaintext document
-        InputStream sourceDocument = readPlaintextDocument();
+        InputStream sourceDocument =
+                this.getClass().getClassLoader().getResourceAsStream(
+                        "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
         DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
         Document document = builder.parse(sourceDocument);
-        
+
         List<String> localNames = new ArrayList<String>();
         localNames.add("PaymentInfo");
-        
+
         sign(XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA512, document, localNames, ecKeyPair.getPrivate());
         // XMLUtils.outputDOM(document, System.out);
         verify(document, ecKeyPair.getPublic(), localNames);
     }
-    
-    private InputStream readPlaintextDocument() throws FileNotFoundException {
+
+    @org.junit.Test
+    public void testECDSA_RIPEMD160() throws Exception {
+        if (!bcInstalled) {
+            return;
+        }
         // Read in plaintext document
-        InputStream sourceDocument = 
+        InputStream sourceDocument =
                 this.getClass().getClassLoader().getResourceAsStream(
                         "ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml");
-        if (sourceDocument == null) {
-            String filename = 
-                "src/test/resources/ie/baltimore/merlin-examples/merlin-xmlenc-five/plaintext.xml";
-            String basedir = System.getProperty("basedir");
-            if (basedir != null && !"".equals(basedir)) {
-                filename = basedir + "/" + filename;
-            }
-            File f = new File(filename);
-            sourceDocument = new FileInputStream(f);
-        }
-        
-        return sourceDocument;
+        DocumentBuilder builder = XMLUtils.createDocumentBuilder(false);
+        Document document = builder.parse(sourceDocument);
+
+        List<String> localNames = new ArrayList<String>();
+        localNames.add("PaymentInfo");
+
+        sign(XMLSignature.ALGO_ID_SIGNATURE_ECDSA_RIPEMD160, document, localNames, ecKeyPair.getPrivate());
+        // XMLUtils.outputDOM(document, System.out);
+        verify(document, ecKeyPair.getPublic(), localNames);
     }
-    
+
     private XMLSignature sign(
         String algorithm,
         Document document,
@@ -388,7 +535,7 @@ public class PKSignatureAlgorithmTest extends org.junit.Assert {
 
         return sig;
     }
-    
+
     private void verify(
         Document document,
         Key key,
